<?php
namespace Controller;
use Util\Instance\GeneralProvider;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Util\Instance\ServiceInstance;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Utilities;
use Exception\CustomException;



class Structure  {

    public function findStructures(DTO\Request\Filter $request, DTO\Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $orderBy = $request->getFilterCriteria()->getOrderBy();
        $Structure = ServiceInstance::getStructure()->findStructure($filters, $page, $pageSize, $orderBy);
        $response->addDataSet(new DataSource\DataSet('Structure', $Structure));
    }

    public function findMtDataBase(DTO\Request\Filter $request, DTO\Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $orderBy = $request->getFilterCriteria()->getOrderBy();
        $DataBase = ServiceInstance::getStructure()->findMtDataBase($filters, $page, $pageSize, $orderBy);
        $response->addDataSet(new DataSource\DataSet('findMtDataBase', $DataBase));
    }

    public function findRepositorioById(Request\Filter $request, Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $repositoiro = ServiceInstance::getStructure()->findRepositorioById($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('repositoiro', $repositoiro));
    }

    public function findOperadorById(Request\Filter $request, Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $operador = ServiceInstance::getStructure()->findOperadorById($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('repositoiro', $operador));
    }


    public function structureScript(DTO\Request\Dataset $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $script = $row['SCRIPT'];
        $connType = $row['connectionType'];
        $connDbId = $row['connectionDataBase'];
        
        try{
            $script = ServiceInstance::getStructure()->structureScript($script, $connType, $connDbId);
            $response->addDataSet(new DataSource\DataSet('scriptResult', $script));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getUserAndRepository(Request\Filter $request, Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $id = $request->getFilterCriteria()->getConditions()[0]["value"];
        $dbId = $request->getFilterCriteria()->getConditions()[1]["value"];
        $header = ServiceInstance::getStructure()->getUserAndRepository($id, $dbId);
        $response->addDataSet(new DataSource\DataSet('logHeader', $header));
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function structureSave(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $currentRow = $row["row"];
            $issueId = $currentRow["ISSUES"];
            $log = array_pop($row["LOG"]);
            $scripts = empty($row["SCRIPTARRAY"]) ? null : $row["SCRIPTARRAY"];
            $script = $currentRow["SCRIPT"];
            $dsDesestrutura= $currentRow["DSDESESTRUTURA"];
            $dsDirLog = empty($log["path"]) ? $currentRow["DSDIRLOG"] : $log["path"];
            $id = empty($currentRow["ID"]) ? null : $currentRow["ID"];
            $idResultScript = $currentRow["IDRESULTSCRIPT"];
            $repositorioId = $currentRow["REPOSITORIO_ID"];
            $idAtivo = $currentRow["IDATIVO"];
            $nrScript = empty($currentRow["NRSCRIPT"]) ? null : $currentRow["NRSCRIPT"];
            $saveType = empty($currentRow["TYPE_SAVE"]) ? null : $currentRow["TYPE_SAVE"];
            $invalidScript = empty($currentRow["INVALID_SCRIPT"]) ? null : $currentRow["INVALID_SCRIPT"];
            $nmObjeto = empty($row["NMOBJETO"]) ? null : $row["NMOBJETO"];
            $mtDb = empty($row["DB_ID"]) ? null : $row["DB_ID"];
            $structureInfo = ServiceInstance::getStructure()->structuresSave($script, $id, $dsDesestrutura, $dsDirLog, $idResultScript,
                    $repositorioId, $idAtivo, $nrScript, $saveType, $invalidScript, $scripts,$issueId, $nmObjeto, $mtDb);
                    
                    
            $response->addDataSet(new DataSource\DataSet('savedScripts', $structureInfo));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function updateScript(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $currentRow = $row["row"];
            $log = array_pop($row["LOG"]);
            $scripts = empty($row["SCRIPTARRAY"]) ? null : $row["SCRIPTARRAY"];
            $dsDirLog = empty($log["path"]) ? $currentRow["DSDIRLOG"] : $log["path"];
            $id = empty($currentRow["ID"]) ? null : $currentRow["ID"];
            $idResultScript = $currentRow["IDRESULTSCRIPT"];
            $repositorioId = $currentRow["REPOSITORIO_ID"];
            $idAtivo = $currentRow["IDATIVO"];
            $nrScript = empty($currentRow["NRSCRIPT"]) ? null : $currentRow["NRSCRIPT"];
            $invalidScript = empty($currentRow["INVALID_SCRIPT"]) ? null : $currentRow["INVALID_SCRIPT"];
            $structureInfo = ServiceInstance::getStructure()->updateScript($scripts, $dsDirLog, $id, $idResultScript, $invalidScript, $idAtivo, $nrScript, $repositorioId);

            $response->addDataSet(new DataSource\DataSet('savedScripts', $structureInfo));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function inactivateScriptStructure(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $id = $row["ID"];
            $structureInfo = ServiceInstance::getStructure()->inactivateScriptStructure($id);

            $response->addDataSet(new DataSource\DataSet('inactivateScriptStructure', [$structureInfo]));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function inactivateScript(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $nmObjeto = $row["NMOBJETO"];
            $connDb = empty($row["DB_ID"]) ? $row["CONN_DB"] : $row["DB_ID"];
            $structureInfo = ServiceInstance::getStructure()->inactivateScript($nmObjeto, $connDb);

            $response->addDataSet(new DataSource\DataSet('inactivateScript', [$structureInfo]));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function relateScripts(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $strucutreIds = $row["STRUCTURES_IDS"];
           
            $structureInfo = ServiceInstance::getStructure()->relateScripts($strucutreIds);

            $response->addDataSet(new DataSource\DataSet('relateScripts', [$structureInfo]));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getScriptNameIsInUse(Request\Filter $request, Response $response)
    {
        
        $name = $request->getFilterCriteria()->getConditions()[0]["value"];
        $id = $request->getFilterCriteria()->getConditions()[1]["value"];
        $connDb = $request->getFilterCriteria()->getConditions()[2]["value"];
        try{
            $result = ServiceInstance::getStructure()->getScriptNameIsInUse($name, $id, $connDb);
            $response->addDataSet(new DataSource\DataSet('ScriptNameIsInUse', $result));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function validateScriptNameInOtherDbs(Request\Filter $request, Response $response)
    {
        
        $name = $request->getFilterCriteria()->getConditions()[0]["value"];
        $dbIds = $request->getFilterCriteria()->getConditions()[1]["value"];
        try{
            $result = ServiceInstance::getStructure()->validateScriptNameInOtherDbs($name, $dbIds);
            $response->addDataSet(new DataSource\DataSet('ScriptNameIsInUse', $result));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getScriptsWithError(Request\Filter $request, Response $response)
    {
        $id = $request->getFilterCriteria()->getConditions()[0]["value"];

        try{
            $script = ServiceInstance::getStructure()->getScriptsWithError($id);
            $response->addDataSet(new DataSource\DataSet('Scripts', $script));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function structureUploadSave(DTO\Request\Dataset $request, DTO\Response $response)
    {

        try {
            $row = $request->getDataSet()->getRows()[0];
            $currentRow = $row["row"];
            $script = $currentRow["SCRIPT"];
            $issueId = $currentRow["ISSUES_UPLOAD"];
            $dsDesestrutura= $currentRow["DSDESESTRUTURA"];
            $idResultScript = "PD";
            $repositorioId = $currentRow["REPOSITORIO_ID"];
            $idAtivo = $currentRow["IDATIVO"];
            $script = ServiceInstance::getStructure()->structureUploadSave($script, $dsDesestrutura, $idResultScript, $repositorioId, $idAtivo,$issueId);
            $response->addDataSet(new DataSource\DataSet('savedScripts', [$script]));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getFullLog(Request\Filter $request, Response $response)
    {
        $dirLog = $request->getFilterCriteria()->getConditions()[0]["value"];
        try {
            $logFileContents = ServiceInstance::getStructure()->getFullLog($dirLog);
            $response->addDataSet(new DataSource\DataSet('logFileContents', [$logFileContents]));
         } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        };
    }

    public function getFullLogs(Request\Filter $request, Response $response)
    {
        $row = $request->getFilterCriteria()->getConditions()[0]["value"];
        try {
            $logFileContents = ServiceInstance::getStructure()->getFullLogs($row);
            $response->addDataSet(new DataSource\DataSet('logFileContents', $logFileContents));
         } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        };
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function downloadScriptLog(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row      = $request->getDataSet()->getRows()[0];
            $fileDir  = $row['FILE_DIR'];
            $fileName = $row['FILE_NAME'];
            ServiceInstance::getStructure()->downloadScriptLog($fileDir, $fileName);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    /**
     * @param DTO\Request\DataSet $request
     * @param DTO\Response $response
     */
    public function downloadScript(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $id  = $row['ID'];
            ServiceInstance::getStructure()->downloadScript($id);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function removeStructure(DTO\Request\DataSet $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];
        $structureId = !empty($row["ID"]) ? $row["ID"] : null;
        $dbIds = !empty($row["DB_ID"]) ? $row["DB_ID"] : null;

        try {
            ServiceInstance::getStructure()->removeStructure($structureId, $dbIds, $row);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function validateWebSocketDirectory(Request\Filter $request, Response $response)
    {
        $idRep = $request->getFilterCriteria()->getConditions()[0]["value"];
        try {
            $dsDirectory = ServiceInstance::getStructure()->validateWebSocketDirectory($idRep);
            $response->addDataSet(new DataSource\DataSet('validateWebSocketDirectory', $dsDirectory));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        };
    }

    public function scriptOnTempFile(DTO\Request\Dataset $request, DTO\Response $response)
    {
        $row = $request->getDataSet()->getRows()[0];

        try {
            $script = array_pop($row["SCRIPT"]);
            $path = $script["path"];
            $return = array();
            $url = Utilities::getFileServerFilesUrlParameter().$path;
            array_push($return, $url, $path);
            $response->addDataSet(new DataSource\DataSet('scriptOnTempFile', $return));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        };
    }

    public function removeScriptOnTempFile(Request\Filter $request, Response $response)
    {
        $imagePath = $request->getFilterCriteria()->getConditions()[0]["value"];
        try {
            ServiceInstance::getStructure()->deleteScriptsInFileServe($imagePath);
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        };
    }
    
    public function findIssues(Request\Filter $request, Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $orderBy = $request->getFilterCriteria()->getOrderBy();
        $repositoiro = ServiceInstance::getStructure()->findIssues($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('findIssues', $repositoiro));
    }
    
    public function findIssuesRelatedToTask(Request\Filter $request, Response $response)
    {
        $page = $request->getFilterCriteria()->getPage();
        $pageSize = $request->getFilterCriteria()->getPageSize();
        $filters = $request->getFilterCriteria()->getConditions();
        $repositoiro = ServiceInstance::getStructure()->findIssuesRelatedToTask($filters, $page, $pageSize);
        $response->addDataSet(new DataSource\DataSet('findIssuesRelatedToTask', $repositoiro));
    }

    public function findRelatedScript(Request\Filter $request, Response $response)
    {
        
        $nrEstrut = $request->getFilterCriteria()->getConditions()[0]["value"];
        $repositoiro = ServiceInstance::getStructure()->findRelatedScript($nrEstrut);
        $response->addDataSet(new DataSource\DataSet('findRelatedScript', $repositoiro));
    }

}

