<?php
namespace Repository;

use Util\Utilities;
use Doctrine\ORM\EntityManager;

class Structure
{

    /** @var EntityManager **/
    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager){
        $this->entityManager = $entityManager;
    }

    public function getAllStructures($conditions, $limit, $offset, $order = '')
    {
        $query = <<<SQL
                SELECT *
                    FROM (SELECT MTD.ID, MTD.SCRIPT, MTD.SCRIPT_SUC, MTD.SCRIPT_ERR, MTD.DSDESESTRUTURA, MTD.DSDIRLOG,
                    (
                            SELECT
                                RTRIM(XMLAGG(XMLELEMENT(E,TAS.TASK,',').EXTRACT('//text()') ORDER BY TAS.TASK).GETCLOBVAL(),',') AS ID
                            FROM MT_DESIGN_ESTRUTURA_TASK TAS
                            WHERE  TAS.MT_DESIGN_ESTRUTURA_ID = MTD.ID
                            GROUP BY TAS.MT_DESIGN_ESTRUTURA_ID
                    ) AS ISSUES,
                    MTD.IDRESULTSCRIPT, MTD.REPOSITORIO_ID, MTD.DTINCLUSAO, MTD.DTULTATU, MTD.CDOPERINCLUSAO, MTD.IDATIVO,
                    MTD.NRSCRIPT, MTR.NMREPOSITORIO, OPI.NMOPERADOR AS OPERINC, OPA.NMOPERADOR AS OPERATT
                    FROM MT_DESIGN_ESTRUTURA MTD
                    JOIN MT_REPOSITORIO MTR ON MTR.ID = MTD.REPOSITORIO_ID
                    JOIN OPERADOR OPI ON MTD.CDOPERINCLUSAO = OPI.CDOPERADOR
                    JOIN OPERADOR OPA ON MTD.CDOPERULTATU = OPA.CDOPERADOR
                    $order)
                    WHERE 1 = 1 
                    $conditions
SQL;

        $query = Utilities::getPaginatedQuery($query, $limit, $offset);

        $returnQuery = $this->entityManager->getConnection()->fetchAll($query);
        foreach ($returnQuery as &$oneQuery) {
            $oneQuery['ISSUES'] = Utilities::setSelectMultiInQuery($oneQuery['ISSUES'], false);
        };
        return $returnQuery;
        
    }

    public function getStructure($conditions,$issueFilter= '',$issueFilterConditions='')
    {
        $query = <<<SQL
                  SELECT *
                    FROM (SELECT MTD.ID, MTD.SCRIPT, MTD.SCRIPT_SUC, MTD.SCRIPT_ERR, MTD.DSDESESTRUTURA, MTD.DSDIRLOG, MTD.NMOBJETO,
                    (
                            SELECT
                                RTRIM(XMLAGG(XMLELEMENT(E,TAS.TASK,',').EXTRACT('//text()') ORDER BY TAS.TASK).GETCLOBVAL(),',') AS ID
                            FROM MT_DESIGN_ESTRUTURA_TASK TAS
                            WHERE  TAS.MT_DESIGN_ESTRUTURA_ID = MTD.ID
                            GROUP BY TAS.MT_DESIGN_ESTRUTURA_ID
                    ) AS ISSUES,
                    (
                            SELECT
                                RTRIM(XMLAGG(XMLELEMENT(E,TAS.TASK,',').EXTRACT('//text()') ORDER BY TAS.TASK).GETCLOBVAL(),',') AS ID
                            FROM MT_DESIGN_ESTRUTURA_TASK TAS
                            WHERE  TAS.MT_DESIGN_ESTRUTURA_ID = MTD.ID
                            $issueFilter
                            GROUP BY TAS.MT_DESIGN_ESTRUTURA_ID
                    ) AS ISSUE_FILTER,
                    MTD.IDRESULTSCRIPT, MTD.REPOSITORIO_ID, MTD.DTINCLUSAO, MTD.DTULTATU, MTD.CDOPERINCLUSAO, MTD.IDATIVO,
                    MTD.NRSCRIPT, MTR.NMREPOSITORIO, OPI.NMOPERADOR AS OPERINC, OPA.NMOPERADOR AS OPERATT, MDB.NAME AS DB_NAME, MTD.MT_BANCO_DADOS_ID AS DB_ID, MTD.NRESTRUTREL
                    FROM MT_DESIGN_ESTRUTURA MTD
                    JOIN MT_REPOSITORIO MTR ON MTR.ID = MTD.REPOSITORIO_ID
                    JOIN OPERADOR OPI ON MTD.CDOPERINCLUSAO = OPI.CDOPERADOR
                    JOIN OPERADOR OPA ON MTD.CDOPERULTATU = OPA.CDOPERADOR
                    LEFT JOIN MT_BANCO_DADOS MDB ON MTD.MT_BANCO_DADOS_ID = MDB.ID
                    )
                    WHERE 1 = 1 
                    $issueFilterConditions
                    $conditions
SQL;
         $returnQuery = $this->entityManager->getConnection()->fetchAll($query);
         foreach ($returnQuery as &$oneQuery) {
            $oneQuery['ISSUES'] = Utilities::setSelectMultiInQuery($oneQuery['ISSUES'], false);
        };
        return $returnQuery;
    }

    public function findIssues($conditions, $limit, $offset)
    {
        $query = <<<SQL
                SELECT NAME,ID
                  FROM TASK
                  WHERE IS_ISSUE = 'Y'
                  AND TASK.STATUS = 'A'
                  AND CLASSIFICATION = 18
SQL;

        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->entityManager->getConnection()->fetchAll($query);
    }

    public function findMtDataBases($conditions, $limit, $offset)
    {
        $query = <<<SQL
                SELECT *
                  FROM MT_BANCO_DADOS
                  WHERE 1=1
                  $conditions
SQL;

        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->entityManager->getConnection()->fetchAll($query);
    }

    public function getDataBaseById($id)
    {
        $query = <<<SQL
                    SELECT * from MT_BANCO_DADOS
                     WHERE ID = $id

SQL;

        return $this->entityManager->getConnection()->fetchAll($query);
    }
    
    public function findIssuesRelatedToTask($conditions,$limit,$offset,$currentUser){
        $query = <<<SQL
                SELECT
                ISS.ID
                FROM TASK TAS
                JOIN ALLOCATION AL ON (TAS.ID = AL.TASK)
                JOIN RESOURCE_T RES ON (RES.ID = AL.RESOURCE_T AND RES.EXTERNAL_CODE = '$currentUser')
                JOIN TASK_RELATIONSHIP TAR ON (TAR.TASK_RELATIONSHIP =TAS.ID AND TAR.TYPE = 'I' )
                JOIN TASK_STATUS TST ON (TST.ID = TAS.TASK_STATUS AND TST.CONTROL = 'D' )
                JOIN TASK ISS ON (ISS.ID = TAR.TASK AND ISS.STATUS = 'A' AND ISS.CLASSIFICATION = 18)
                WHERE TAS.IS_ISSUE = 'N'
                AND TAS.STATUS = 'A'
                AND TAS.PERCENT_COMPLETED <> 100
                $conditions
        
SQL;
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        $dataset = $this->entityManager->getConnection()->fetchAll($query);
        $arrayToReturn = [];
        foreach($dataset as $array){
            array_push($arrayToReturn, $array['ID']);
        }
        return $arrayToReturn;
        
    }

    public function findRelatedScript($estrut){
        $query = <<<SQL
                SELECT
                MDE.MT_BANCO_DADOS_ID AS DB_ID, MDE.*, MBD.NAME
                FROM MT_DESIGN_ESTRUTURA  MDE
                JOIN MT_BANCO_DADOS MBD ON MDE.MT_BANCO_DADOS_ID = MBD.ID
                WHERE NRESTRUTREL = (SELECT NRESTRUTREL 
                                       FROM MT_DESIGN_ESTRUTURA
                                       WHERE ID = $estrut)
                
        
SQL;
        return $this->entityManager->getConnection()->fetchAll($query);
        
    }

    
    public function findRepositorioById($conditions, $limit, $offset)
    {
        $query = <<<SQL
                SELECT ID, NMREPOSITORIO
                  FROM MT_REPOSITORIO
                 WHERE IDATIVO = 'S'
                       $conditions

SQL;
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->entityManager->getConnection()->fetchAll($query);


    }
    public function findRepositorioByIdNoPagination($conditions)
    {
        $query = <<<SQL
                SELECT NMREPOSITORIO
                  FROM MT_REPOSITORIO
                 WHERE IDATIVO = 'S'
                       $conditions

SQL;
        return $this->entityManager->getConnection()->fetchAll($query);


    }

    public function getScriptNameIsInUse($name, $id, $connDb)
    {
        $query = <<<SQL
                SELECT ID, NMOBJETO
                  FROM MT_DESIGN_ESTRUTURA
                 WHERE NMOBJETO =  '$name' AND IDATIVO = 'S' AND ID != $id AND MT_BANCO_DADOS_ID = $connDb

SQL;
        return $this->entityManager->getConnection()->fetchAll($query);


    }

    public function validateScriptNameInOtherDbs($name, $dbIds)
    {
        $query = <<<SQL
            SELECT * FROM MT_BANCO_DADOS 
             WHERE ID IN (SELECT MT_BANCO_DADOS_ID 
                            FROM MT_DESIGN_ESTRUTURA 
                           WHERE NMOBJETO = '$name' AND IDATIVO = 'S' AND MT_BANCO_DADOS_ID NOT IN ($dbIds))

                

SQL;
        return $this->entityManager->getConnection()->fetchAll($query);


    }

    public function findScriptsWithError($id)
    {
        $query = <<<SQL
                SELECT MTS.ID, MTE.MT_BANCO_DADOS_ID AS DB_ID, MTB.NAME AS DB_NAME, MTS.SCRIPT
                  FROM MT_DESIGN_ESTRUTURA_SCRIPT MTS
                  JOIN MT_DESIGN_ESTRUTURA MTE ON MTS.MT_DESIGN_ESTRUT_ID = MTE.ID
                  JOIN MT_BANCO_DADOS MTB ON MTE.MT_BANCO_DADOS_ID = MTB.ID
                 WHERE MT_DESIGN_ESTRUT_ID = $id
                   AND STATUS = 'E'
              ORDER BY SCRIPT_ORDER

SQL;
        return $this->entityManager->getConnection()->fetchAll($query);


    }

    public function findAllrelatedScripts($id)
    {
        $query = <<<SQL
                SELECT ID
                  FROM MT_DESIGN_ESTRUTURA
                WHERE NRESTRUTREL IN (   SELECT NRESTRUTREL
                                  FROM MT_DESIGN_ESTRUTURA
                                 WHERE ID = $id)
                AND ID <> $id
                UNION
                SELECT ID FROM MT_DESIGN_ESTRUTURA
                WHERE ID = $id    

SQL;
        return $this->entityManager->getConnection()->fetchAll($query);


    }

    public function getOrderlyScriptsById($id)
    {
        $query = <<<SQL
               SELECT SCRIPT
                 FROM MT_DESIGN_ESTRUTURA_SCRIPT
                WHERE MT_DESIGN_ESTRUT_ID = $id
             ORDER BY SCRIPT_ORDER ASC , ID ASC

SQL;
        return $this->entityManager->getConnection()->fetchAll($query);


    }

    // just active ones
    public function findOperadorById($conditions, $limit, $offset)
    {
        $query = <<<SQL
                SELECT CDOPERADOR, NMOPERADOR
                  FROM OPERADOR
                 WHERE IDATIVO = 'S'
                       $conditions

SQL;
        $query = Utilities::getPaginatedQuery($query, $limit, $offset);
        return $this->entityManager->getConnection()->fetchAll($query);


    }

}