<?php

namespace Util\ORM;

use Doctrine\DBAL\Platforms\AbstractPlatform;
use Doctrine\DBAL\Types\ConversionException;
use Doctrine\DBAL\Types\DateTimeType as DoctrineDateTimeType;
use Util\ORM\DateTime as SetformsDateTime;

class DateTimeType extends DoctrineDateTimeType
{
    const DATE_FORMAT = "d/m/Y H:i:s";

    public function convertToPHPValue($value, AbstractPlatform $platform)
    {
        if ($value === null || $value instanceof SetformsDateTime) {
            return $value;
        }

        $formatString = self::DATE_FORMAT;
        $val = SetformsDateTime::createFromFormat($formatString, $value);
        if ( ! $val) {
            throw ConversionException::conversionFailedFormat($value, $this->getName(), $formatString);
        }

        return $val;
    }

    public function convertToDatabaseValue($value, AbstractPlatform $platform)
    {
        return ($value !== null)
            ? $value->format(self::DATE_FORMAT) : null;
    }
} 