-- Sistemas Integrados TEKNISA
-- Faturamento de Servios
--
-- Nome: 0001_FASSTPO_FATURA_MANUTENCAO.sql
--
-- Descrio:
-- Procedimento para apurao das mensalidades (manuteno e locao) das linhas de produto
-- dos clientes da Teknisa.
--
-- Parmetros:
-- P_CLIENTE - Cdigo do cliente
-- P_CDOPERAPUR - Cdigo do operador que comandou a apurao
-- P_DTMESAPUR - Ms a ser apurado
-- P_NAC - Identificador de faturamento para clientes internacionais ('_I_') ou nacionais ('_N_')
--
-- Criao:
-- 15/07/2010 Ederson Melquades da Cunha

create or replace PROCEDURE FATURA_MANULOCA
  (P_CLIENTE IN VARCHAR2,
   P_CDOPERAPUR IN APURFATUFAS.CDOPERAPUR%TYPE,
   P_DTMESAPUR IN DATE,
   P_NAC IN VARCHAR2)
AS
   V_DTAPURACAO DATE;
   V_DTHRAPURACAO DATE;
   V_DTMES_APURACAO DATE;
   V_DTULTIMO_MES_APURADO DATE;

   V_CDPRODUTO_NACIONAL PRODUTO.CDPRODUTO%TYPE;
   V_CDPRODUTO_INTERNACIONAL PRODUTO.CDPRODUTO%TYPE;

   V_DSBOLETCLIE PARCLIEFAS.DSBOLETCLIE%TYPE;
   V_DSFATURCLIE PARCLIEFAS.DSFATURCLIE%TYPE;
   V_CLIENTE_IDFATURAPFILI PARCLIEFAS.IDFATURAPFILI%TYPE;
   V_CDEMPRESA EMPRESA.CDEMPRESA%TYPE;
   V_CDFILIAL FILIAL.CDFILIAL%TYPE;

   V_CDPRODUTO PRODUTO.CDPRODUTO%TYPE;

   V_NRAPURFATFAS VARCHAR2(20);

   V_QTLICENCAS_VALIDAS NUMBER;
   V_QTLICENCAS_JA_FATURADAS NUMBER;
   V_QTLICENCAS_A_FATURAR NUMBER;

   V_GRAVOU_CONTRATOLINPAPU CHAR(1) ;
   V_DESCLINHAPROD CHAR(1) ;

   V_APURAR_UNIDADES_AGRUPADAS CHAR(1) ;
   V_NRAPURFATFAS_AGRUPADA VARCHAR2(10);

   V_VRTOTAFATFAS APURFATUFAS.VRTOTAFATFAS%TYPE;

   V_MENSAGEM VARCHAR2(2000);
   V_SISTEMAS VARCHAR2(2000);
   V_DSOPERLUSO LOGLUSO.DSOPERLUSO%TYPE;
   V_DSOPERLINP LOGLINP.DSOPERLINP%TYPE;

   V_LINPLUSO_VALIDA BOOLEAN;
   V_QTCAIXASFILIAL NUMBER;

BEGIN

   SELECT TRUNC(TO_DATE(SYSDATE)), SYSDATE
     INTO V_DTAPURACAO, V_DTHRAPURACAO
     FROM DUAL;

   SELECT MAX(CDPROFATMANU), MAX(CDPROFATINTMANU)
     INTO V_CDPRODUTO_NACIONAL, V_CDPRODUTO_INTERNACIONAL
     FROM PAR_GERAL_FAS;

   SELECT MAX(A.DTMESAPURFAS)
     INTO V_DTULTIMO_MES_APURADO
     FROM APURFATUFAS A
    WHERE A.IDAPURFATFAS = '6'  -- MENSALIDADE (MANUTENO OU LOCAO)
      AND A.IDTIPOLANCAPUR = 'APURACAO';

   V_DTMES_APURACAO := TO_DATE('01' || TO_CHAR(P_DTMESAPUR, 'MM') || TO_CHAR(P_DTMESAPUR, 'YYYY'), 'DDMMYYYY');

   -- ###################################################################################
   -- CURSOR DE CLIENTES: IGNORA CLIENTES QUE POSSUAM APURAES COM NF OU INVOICE GERADOS
   -- ###################################################################################
   FOR CUR_CLIENTE IN (SELECT C.CDCLIENTE, C.NMFANTCLIE, C.IDIDIOMA, E.CDPAIS
                         FROM CLIENTE C, ENDECLIE E
                        WHERE ((INSTR(P_CLIENTE, '_' || C.CDCLIENTE || '_') > 0) OR (P_CLIENTE = 'C'))
                          AND E.CDCLIENTE = C.CDCLIENTE
                          AND E.IDTPENDECLIE = 'P'
                          AND ((E.CDPAIS <> '0055' AND (INSTR(P_NAC, '_I_') > 0)) OR (E.CDPAIS = '0055' AND (INSTR(P_NAC, '_N_') > 0)))
                          AND NOT EXISTS (SELECT A.*
                                            FROM APURFATUFAS A
                                           WHERE A.CDCLIENTE = C.CDCLIENTE
                                             AND A.IDAPURFATFAS = '6' -- MENSALIDADE (MANUTENO OU LOCAO)
                                             AND A.DTMESAPURFAS = V_DTMES_APURACAO
                                             AND A.IDTIPOLANCAPUR = 'APURACAO'
                                             AND A.IDSTAPFATFAS = 'R') -- COM NOTA FISCAL OU INVOICE GERADO
                          AND EXISTS (SELECT L.*
                                        FROM CONTRATOLINP L
                                       WHERE L.CDCLIENTE = C.CDCLIENTE
                                         AND L.DTCONTLINPRESC IS NULL) -- SOMENTE CLIENTES COM LINHAS DE PRODUTO COM CONTRATO ATIVO
                        ORDER BY C.NMFANTCLIE) LOOP

      -- ################################################################################
      -- EXCLUSO DE LINHAS DE PRODUTO E CAIXAS EXCLUDOS PELO USURIO NO SITE E VENCIDOS
      -- ################################################################################

      -- EXCLUI CAIXAS EXCLUDOS PELO USURIO NO SITE, PAGOS E VENCIDOS
      DELETE
        FROM CAIXALUSO
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND IDCAIXAEXCLUIDO = 'S'
         AND DTVALPERSCXPAGO IS NOT NULL
         AND DTVALPERSCXPAGO < TRUNC(V_DTHRAPURACAO);
      -- EXCLUI CAIXAS EXCLUDOS PELO USURIO NO SITE, NO PAGOS, VENCIDOS A 60 DIAS OU MAIS
      DELETE
        FROM CAIXALUSO
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND IDCAIXAEXCLUIDO = 'S'
         AND DTVALPERSCXPEND IS NOT NULL
         AND (DTVALPERSCXPEND + 60) < TRUNC(V_DTHRAPURACAO);

      FOR CUR_EXCLUI_LINPLUSO IN (SELECT C.CDCLIENTE, C.CDLINPCLIE, C.IDTIPLINPC
                                    FROM CONTRATOLINP C
                                   WHERE C.CDCLIENTE = CUR_CLIENTE.CDCLIENTE
                                     AND C.IDTIPLINPC IN ('M', 'L')) LOOP
         -- EXCLUI LINHAS DE PRODUTO EXCLUDAS PELO USURIO NO SITE, PAGAS E VENCIDAS
         DELETE
           FROM LINPLUSO
          WHERE CDCLIENTE = CUR_EXCLUI_LINPLUSO.CDCLIENTE
            AND CDLINPCLIE = CUR_EXCLUI_LINPLUSO.CDLINPCLIE
            AND IDLINPEXCLUIDA = 'S'
            AND DTVALPERSLUPAGA IS NOT NULL
            AND DTVALPERSLUPAGA < TRUNC(V_DTHRAPURACAO);
         -- EXCLUI LINHAS DE PRODUTO EXCLUDAS PELO USURIO NO SITE, NO PAGAS, VENCIDAS A 60 DIAS OU MAIS
         DELETE
           FROM LINPLUSO
          WHERE CDCLIENTE = CUR_EXCLUI_LINPLUSO.CDCLIENTE
            AND CDLINPCLIE = CUR_EXCLUI_LINPLUSO.CDLINPCLIE
            AND IDLINPEXCLUIDA = 'S'
            AND DTVALPERSLUPEND IS NOT NULL
            AND (DTVALPERSLUPEND + 60) < TRUNC(V_DTHRAPURACAO);
      END LOOP;

      -- #######################################################################
      -- EXCLUI APURAES QUE J TENHAM SIDO REALIZADAS E REGISTROS DE HISTRICO
      -- #######################################################################

      DELETE
        FROM APURFATUFAS
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND IDAPURFATFAS = '6' -- MENSALIDADE (MANUTENO OU LOCAO)
         AND DTMESAPURFAS = V_DTMES_APURACAO
         AND IDTIPOLANCAPUR = 'APURACAO';

      DELETE
        FROM CAIXALUSOAPUR
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND DTMESAPURLINP = V_DTMES_APURACAO;

      DELETE
        FROM LINPLUSOAPUR
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND DTMESAPURLINP = V_DTMES_APURACAO;

      DELETE
        FROM SISTCONTLINPAPU
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND DTMESAPURLINP = V_DTMES_APURACAO;

      DELETE
        FROM CONTRATOLINPAPU
       WHERE CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND DTMESAPURLINP = V_DTMES_APURACAO;

      -- #######################################################################
      -- #######################################################################
      -- #######################################################################

      SELECT MAX(P.DSBOLETCLIE), MAX(P.DSFATURCLIE), NVL(MAX(P.IDFATURAPFILI),'N'), NVL(MAX(P.CDEMPRESA),'01'), NVL(MAX(F.CDFILIAL),'9999')
        INTO V_DSBOLETCLIE, V_DSFATURCLIE, V_CLIENTE_IDFATURAPFILI, V_CDEMPRESA, V_CDFILIAL
        FROM PARCLIEFAS P, FILIAL F
       WHERE P.CDCLIENTE = CUR_CLIENTE.CDCLIENTE
         AND P.IDTPFATUCLIE = '3' -- MENSALIDADE (MANUTENO OU LOCAO)
         AND F.CDEMPRESA = P.CDEMPRESA;

      /*A EMPRESA 12 POSSUI 3 FILIAIS, A PEDIDO DA POLIANA SER GERADO O FATURAMENTO SEMPRE PARA A FILIAL 9990*/ 
      IF V_CDEMPRESA = '12' THEN
         V_CDFILIAL := '9990';
      END IF;

      IF INSTR(V_DSBOLETCLIE, '[MM/YYYY]') > 0 THEN
         V_DSBOLETCLIE := SUBSTR(V_DSBOLETCLIE, 1, INSTR(V_DSBOLETCLIE, '[MM/YYYY]') -1) ||
                          TO_CHAR(V_DTMES_APURACAO, 'MM/YYYY') ||
                          SUBSTR(V_DSBOLETCLIE, INSTR(V_DSBOLETCLIE, '[MM/YYYY]') + 9, LENGTH(V_DSBOLETCLIE));
      END IF;

      V_DESCLINHAPROD := 'N';

      IF INSTR(V_DSBOLETCLIE, '[LINHAPROD]') > 0 THEN
         V_DESCLINHAPROD := 'S';
         V_DSBOLETCLIE   := SUBSTR(V_DSBOLETCLIE, 1, INSTR(V_DSBOLETCLIE, '[LINHAPROD]') -1);
      END IF;

      IF CUR_CLIENTE.CDPAIS = '0055' THEN
         V_CDPRODUTO := V_CDPRODUTO_NACIONAL;
      ELSE
         V_CDPRODUTO := V_CDPRODUTO_INTERNACIONAL;
      END IF;

      -- ######################################
      -- ######################################
      -- CURSOR DE LINHAS DE PRODUTO DO CLIENTE
      -- ######################################
      -- ######################################
      FOR CUR_LINP IN (SELECT CDCLIENTE, CDLINPCLIE, NMLINPCLIE,
                              IDTIPLINPC, NRUSUALINPC, IDMPROCLINPC, IDLPROCLINPC,
                              VRLICENCUSOLINP, QTMINIMAFATLINP, VRMENSALIDALINP,
                              CDGRUPONFLINPC, QTATIVAS, QTVALIDAS
                         FROM TABLE (LINHA_PRODUTO_FAT (CUR_CLIENTE.CDCLIENTE, V_DTMES_APURACAO)) T
                        WHERE T.CDLINPCLIE <> 'XXX'
                          AND T.VRMENSALIDALINP > 0
                          AND (T.QTMINIMAFATLINP + T.QTATIVAS + T.QTVALIDAS) > 0
                          AND NOT EXISTS (SELECT C.CDCLIENTE
                                            FROM CARENCIALINP C
                                           WHERE C.CDCLIENTE = T.CDCLIENTE
                                             AND C.CDLINPCLIE = T.CDLINPCLIE
                                             AND C.DTMESCARENCLINP = V_DTMES_APURACAO)
                        ORDER BY NMLINPCLIE) LOOP

         V_QTLICENCAS_VALIDAS := 0;
         V_QTLICENCAS_JA_FATURADAS := 0;

         V_GRAVOU_CONTRATOLINPAPU := 'N';
         V_APURAR_UNIDADES_AGRUPADAS := 'N';

         IF CUR_CLIENTE.CDPAIS = '0055' THEN
            V_SISTEMAS := ' - ' || CUR_LINP.NMLINPCLIE || '.';
         ELSE
            IF CUR_CLIENTE.IDIDIOMA = 'ING' THEN
               V_SISTEMAS := ' -  ' || CUR_LINP.NMLINPCLIE || '.';
            ELSE
               V_SISTEMAS := ' -  ' || CUR_LINP.NMLINPCLIE || '.';
            END IF;
         END IF;

         -- ##################################################
         -- CURSOR DE UNIDADES POR LINHA DE PRODUTO DO CLIENTE
         -- ##################################################
         FOR CUR_UNIDADE IN (SELECT LL.NRLICENCAUSO, LU.NMFANTLUSO, LU.IDFATURAFILISEP,
                                    LU.CDLOCALUSO, LU.NMRAZSOCLUSO, LU.NMCONTLUSO,
                                    LU.NRTELELUSO, LU.IDLUSOATIV, LU.DTLIBEPERSLUSO, LU.DTVALIPERSLUSO,
                                    LL.DTVALPERSLUPEND, LL.DTVALPERSLUPAGA, LL.IDLINPEXCLUIDA
                               FROM LICENCAUSO LU, LINPLUSO LL
                              WHERE LL.CDCLIENTE = CUR_LINP.CDCLIENTE
                                AND LL.CDLINPCLIE = CUR_LINP.CDLINPCLIE
                                AND LU.CDCLIENTE = LL.CDCLIENTE
                                AND LU.NRLICENCAUSO = LL.NRLICENCAUSO
                              ORDER BY LL.NRLICENCAUSO) LOOP

            V_LINPLUSO_VALIDA := (SITUACAO_LINPLUSO (CUR_UNIDADE.IDLUSOATIV, CUR_UNIDADE.DTVALPERSLUPEND, CUR_UNIDADE.DTVALPERSLUPAGA, CUR_UNIDADE.IDLINPEXCLUIDA, V_DTMES_APURACAO, 'VALIDA') = 'VALIDA');

            IF CUR_LINP.IDTIPLINPC IN ('M', 'L') THEN
               -- LINHA DE PRODUTO DO TIPO "PRODUTO" (SISTEMAS) SOMA UMA LICENA POR UNIDADE
               IF V_LINPLUSO_VALIDA THEN
                  V_QTLICENCAS_VALIDAS := V_QTLICENCAS_VALIDAS + 1;
               END IF;
            ELSE
               -- LINHA DE PRODUTO DO TIPO "PONTO DE ACESSO" (CAIXAS) SOMA UMA LICENA PARA CADA CAIXA DA FILIAL
               V_QTCAIXASFILIAL := 0;
               SELECT COUNT(*)
                 INTO V_QTCAIXASFILIAL
                 FROM CAIXALUSO CX
                WHERE CX.CDCLIENTE = CUR_LINP.CDCLIENTE
                  AND CX.NRLICENCAUSO = CUR_UNIDADE.NRLICENCAUSO
                  AND SITUACAO_LINPLUSO (CUR_UNIDADE.IDLUSOATIV, CX.DTVALPERSCXPEND, CX.DTVALPERSCXPAGO, CX.IDCAIXAEXCLUIDO, V_DTMES_APURACAO, 'VALIDA') = 'VALIDA';
               V_QTLICENCAS_VALIDAS := V_QTLICENCAS_VALIDAS + V_QTCAIXASFILIAL;
            END IF;

            IF V_CLIENTE_IDFATURAPFILI = 'S' OR CUR_UNIDADE.IDFATURAFILISEP = 'S' THEN
            -- VERIFICA SE DEVE SER APURADO SEPARADAMENTE POR UNIDADE DE ACORDO COM O PARMETRO DO CLIENTE OU DA UNIDADE
               IF CUR_LINP.IDTIPLINPC IN ('M', 'L') THEN
                  IF V_LINPLUSO_VALIDA THEN
                     V_QTLICENCAS_JA_FATURADAS := V_QTLICENCAS_JA_FATURADAS + 1;
                  END IF;
               ELSE
                  V_QTLICENCAS_JA_FATURADAS := V_QTLICENCAS_JA_FATURADAS + V_QTCAIXASFILIAL;
               END IF;

               IF CUR_LINP.IDTIPLINPC IN ('M', 'L') THEN
                  IF CUR_CLIENTE.CDPAIS = '0055' THEN
                     V_MENSAGEM := ' - Unidade ';
                  ELSE
                     IF CUR_CLIENTE.IDIDIOMA = 'ING' THEN
                        V_MENSAGEM := ' - Unidade ';
                     ELSE
                        V_MENSAGEM := ' - Unidade ';
                     END IF;
                  END IF;
               ELSE
                  IF CUR_CLIENTE.CDPAIS = '0055' THEN
                     V_MENSAGEM := ' - Ponto Acesso Unidade ';
                  ELSE
                     IF CUR_CLIENTE.IDIDIOMA = 'ING' THEN
                        V_MENSAGEM := ' - Ponto Acesso Unidade ';
                     ELSE
                        V_MENSAGEM := ' - Ponto Acesso Unidade ';
                     END IF;
                  END IF;
               END IF;

               V_VRTOTAFATFAS := 0;
               IF CUR_LINP.IDTIPLINPC IN ('M', 'L') THEN
                  IF V_LINPLUSO_VALIDA THEN
                     V_VRTOTAFATFAS := CUR_LINP.VRMENSALIDALINP;
                  END IF;
               ELSE
                  V_VRTOTAFATFAS := ROUND(V_QTCAIXASFILIAL * CUR_LINP.VRMENSALIDALINP, 2);
               END IF;

               IF V_VRTOTAFATFAS > 0 THEN
                  NOVO_CODIGO('APURFATUFAS', V_NRAPURFATFAS);
                  V_NRAPURFATFAS := SUBSTR(V_NRAPURFATFAS, 11, 10);

                  INSERT INTO APURFATUFAS
                    (CDFILIAL, CDCLIENTE, NRAPURFATFAS,
                     DTAPURFATFAS, VRTOTAFATFAS, DSEMISSPRONF,
                     CDOPERAPUR, CDEMPRESA, CDPRODUTO,
                     IDAPURFATFAS, IDSTAPFATFAS, IDCOMIFATFAS,
                     DSBOLEFATFAS, DTMESAPURFAS, CDLINPCLIE,
                     CDGRUPONFAPUR, NRLICENCAUSO, IDFORMFATURAMEN)
                  VALUES
                    (V_CDFILIAL, CUR_LINP.CDCLIENTE, V_NRAPURFATFAS,
                     V_DTAPURACAO, V_VRTOTAFATFAS, V_DSFATURCLIE || V_MENSAGEM || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO,
                     P_CDOPERAPUR, V_CDEMPRESA, V_CDPRODUTO,
                     '6', 'P', 'N',
                     DECODE(V_DESCLINHAPROD, 'S', V_DSBOLETCLIE || V_SISTEMAS || V_MENSAGEM || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO, V_DSBOLETCLIE),
                     V_DTMES_APURACAO, CUR_LINP.CDLINPCLIE,
                     CUR_UNIDADE.CDLOCALUSO, CUR_UNIDADE.NRLICENCAUSO, DECODE(V_CLIENTE_IDFATURAPFILI, 'S', 'F', DECODE(CUR_UNIDADE.IDFATURAFILISEP, 'S', 'S', 'N')));

                  IF V_GRAVOU_CONTRATOLINPAPU = 'N' THEN
                     V_GRAVOU_CONTRATOLINPAPU := 'S';
                     INSERT INTO CONTRATOLINPAPU
                       (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NMLINPCLIEAPUR,
                        NRUSUALINPAPUR, IDMPROCLINPAPUR, IDLPROCLINPAPUR, IDTIPLINPCAPUR,
                        QTMINIMAFATAPUR, VRLICENCUSOAPUR, VRMENSALIDAAPUR)
                     VALUES
                       (CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_LINP.NMLINPCLIE,
                        CUR_LINP.NRUSUALINPC, CUR_LINP.IDMPROCLINPC, CUR_LINP.IDLPROCLINPC, CUR_LINP.IDTIPLINPC,
                        CUR_LINP.QTMINIMAFATLINP, CUR_LINP.VRLICENCUSOLINP, CUR_LINP.VRMENSALIDALINP);

                     INSERT INTO SISTCONTLINPAPU
                       (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, CDSISTEMA)
                     SELECT SC.CDCLIENTE, SC.CDLINPCLIE, V_DTMES_APURACAO, SC.CDSISTEMA
                       FROM SISTCONTLINP SC
                      WHERE SC.CDCLIENTE = CUR_LINP.CDCLIENTE
                        AND SC.CDLINPCLIE = CUR_LINP.CDLINPCLIE;
                  END IF;

                  INSERT INTO LINPLUSOAPUR
                    (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NRLICENCAUSO,
                     CDLOCALUSOAPUR, NMFANTLUSOAPUR, NMRAZSOCLUSOAPU, NMCONTLUSOAPUR,
                     NRTELELUSOAPUR, IDLUSOATIVAPUR, DTLIBEPERSAPUR, DTVALIPERSAPUR)
                  VALUES
                    (CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_UNIDADE.NRLICENCAUSO,
                     CUR_UNIDADE.CDLOCALUSO, CUR_UNIDADE.NMFANTLUSO, CUR_UNIDADE.NMRAZSOCLUSO, CUR_UNIDADE.NMCONTLUSO,
                     CUR_UNIDADE.NRTELELUSO, CUR_UNIDADE.IDLUSOATIV, CUR_UNIDADE.DTLIBEPERSLUSO, CUR_UNIDADE.DTVALIPERSLUSO);

                  IF CUR_LINP.IDTIPLINPC IN ('P', 'A') THEN
                     INSERT INTO CAIXALUSOAPUR
                       (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NRLICENCAUSO,
                        CDCAIXALUSOAPUR, NMCAIXALUSOAPUR, DTVALIPERCXAPUR)
                     SELECT CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_UNIDADE.NRLICENCAUSO,
                            CL.CDCAIXALUSO, CL.NMCAIXALUSO, CUR_UNIDADE.DTVALIPERSLUSO
                       FROM CAIXALUSO CL
                      WHERE CL.CDCLIENTE = CUR_LINP.CDCLIENTE
                        AND CL.NRLICENCAUSO = CUR_UNIDADE.NRLICENCAUSO
                        AND SITUACAO_LINPLUSO (CUR_UNIDADE.IDLUSOATIV, CL.DTVALPERSCXPEND, CL.DTVALPERSCXPAGO, CL.IDCAIXAEXCLUIDO, V_DTMES_APURACAO, 'VALIDA') = 'VALIDA';
                  END IF;
               END IF;
            ELSE
            -- APURA TODAS AS UNIDADES AGRUPADAS PARA A LINHA DE PRODUTO
               IF (CUR_LINP.IDTIPLINPC IN ('M', 'L') AND V_LINPLUSO_VALIDA) OR (CUR_LINP.IDTIPLINPC IN ('P', 'A') AND V_QTCAIXASFILIAL > 0) THEN
                  IF V_APURAR_UNIDADES_AGRUPADAS = 'N' THEN
                     V_APURAR_UNIDADES_AGRUPADAS := 'S';

                     NOVO_CODIGO('APURFATUFAS', V_NRAPURFATFAS);
                     V_NRAPURFATFAS := SUBSTR(V_NRAPURFATFAS, 11, 10);

                     V_NRAPURFATFAS_AGRUPADA := V_NRAPURFATFAS;

                     INSERT INTO APURFATUFAS
                       (CDFILIAL, CDCLIENTE, NRAPURFATFAS,
                        DTAPURFATFAS, VRTOTAFATFAS, DSEMISSPRONF,
                        CDOPERAPUR, CDEMPRESA,CDPRODUTO,
                        IDAPURFATFAS, IDSTAPFATFAS, IDCOMIFATFAS,
                        DSBOLEFATFAS, DTMESAPURFAS, CDLINPCLIE,
                        CDGRUPONFAPUR, NRLICENCAUSO, IDFORMFATURAMEN)
                     VALUES
                       (V_CDFILIAL, CUR_LINP.CDCLIENTE, V_NRAPURFATFAS_AGRUPADA,
                        V_DTAPURACAO, 0, V_DSFATURCLIE,
                        P_CDOPERAPUR, V_CDEMPRESA, V_CDPRODUTO,
                        '6', 'P', 'N',
                        DECODE(V_DESCLINHAPROD, 'S', V_DSBOLETCLIE || V_SISTEMAS, V_DSBOLETCLIE),
                        V_DTMES_APURACAO, CUR_LINP.CDLINPCLIE,
                        CUR_LINP.CDGRUPONFLINPC, NULL, DECODE(V_CLIENTE_IDFATURAPFILI, 'S', 'S', 'N'));

                     IF V_GRAVOU_CONTRATOLINPAPU = 'N' THEN
                        V_GRAVOU_CONTRATOLINPAPU := 'S';
                        INSERT INTO CONTRATOLINPAPU
                          (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NMLINPCLIEAPUR,
                           NRUSUALINPAPUR, IDMPROCLINPAPUR, IDLPROCLINPAPUR, IDTIPLINPCAPUR,
                           QTMINIMAFATAPUR, VRLICENCUSOAPUR, VRMENSALIDAAPUR)
                        VALUES
                          (CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_LINP.NMLINPCLIE,
                           CUR_LINP.NRUSUALINPC, CUR_LINP.IDMPROCLINPC, CUR_LINP.IDLPROCLINPC, CUR_LINP.IDTIPLINPC,
                           CUR_LINP.QTMINIMAFATLINP, CUR_LINP.VRLICENCUSOLINP, CUR_LINP.VRMENSALIDALINP);

                        INSERT INTO SISTCONTLINPAPU
                          (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, CDSISTEMA)
                        SELECT SC.CDCLIENTE, SC.CDLINPCLIE, V_DTMES_APURACAO, SC.CDSISTEMA
                          FROM SISTCONTLINP SC
                         WHERE SC.CDCLIENTE = CUR_LINP.CDCLIENTE
                           AND SC.CDLINPCLIE = CUR_LINP.CDLINPCLIE;
                     END IF;
                  END IF;

                  INSERT INTO LINPLUSOAPUR
                    (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NRLICENCAUSO,
                     CDLOCALUSOAPUR, NMFANTLUSOAPUR, NMRAZSOCLUSOAPU, NMCONTLUSOAPUR,
                     NRTELELUSOAPUR, IDLUSOATIVAPUR, DTLIBEPERSAPUR, DTVALIPERSAPUR)
                  VALUES
                    (CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_UNIDADE.NRLICENCAUSO,
                     CUR_UNIDADE.CDLOCALUSO, CUR_UNIDADE.NMFANTLUSO, CUR_UNIDADE.NMRAZSOCLUSO, CUR_UNIDADE.NMCONTLUSO,
                     CUR_UNIDADE.NRTELELUSO, CUR_UNIDADE.IDLUSOATIV, CUR_UNIDADE.DTLIBEPERSLUSO, CUR_UNIDADE.DTVALIPERSLUSO);

                  IF CUR_LINP.IDTIPLINPC IN ('P', 'A') THEN
                     INSERT INTO CAIXALUSOAPUR
                       (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NRLICENCAUSO,
                        CDCAIXALUSOAPUR, NMCAIXALUSOAPUR, DTVALIPERCXAPUR)
                     SELECT CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_UNIDADE.NRLICENCAUSO,
                            CL.CDCAIXALUSO, CL.NMCAIXALUSO, CUR_UNIDADE.DTVALIPERSLUSO
                       FROM CAIXALUSO CL
                      WHERE CL.CDCLIENTE = CUR_LINP.CDCLIENTE
                        AND CL.NRLICENCAUSO = CUR_UNIDADE.NRLICENCAUSO
                        AND SITUACAO_LINPLUSO (CUR_UNIDADE.IDLUSOATIV, CL.DTVALPERSCXPEND, CL.DTVALPERSCXPAGO, CL.IDCAIXAEXCLUIDO, V_DTMES_APURACAO, 'VALIDA') = 'VALIDA';
                  END IF;
               END IF;
            END IF;

            IF CUR_LINP.IDTIPLINPC IN ('M', 'L') THEN
               IF CUR_CLIENTE.CDPAIS = '0055' THEN
                  SELECT 'Apurao do valor mensal de ' || DECODE (CUR_LINP.IDTIPLINPC, 'M', 'manuteno', 'locao') INTO V_DSOPERLUSO FROM DUAL;
                  V_DSOPERLINP := V_DSOPERLUSO || ' da unidade ' || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO;
               ELSE
                  IF CUR_CLIENTE.IDIDIOMA = 'ING' THEN
                     SELECT 'Apurao do valor mensal de ' || DECODE (CUR_LINP.IDTIPLINPC, 'M', 'manuteno', 'locao') INTO V_DSOPERLUSO FROM DUAL;
                     V_DSOPERLINP := V_DSOPERLUSO || ' da unidade ' || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO;
                  ELSE
                     SELECT 'Apurao do valor mensal de ' || DECODE (CUR_LINP.IDTIPLINPC, 'M', 'manuteno', 'locao') INTO V_DSOPERLUSO FROM DUAL;
                     V_DSOPERLINP := V_DSOPERLUSO || ' da unidade ' || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO;
                  END IF;
               END IF;
            ELSE
               IF CUR_CLIENTE.CDPAIS = '0055' THEN
                  SELECT 'Apurao do valor mensal de '|| DECODE (CUR_LINP.IDTIPLINPC, 'M', 'manuteno', 'locao') INTO V_DSOPERLUSO FROM DUAL;
                  V_DSOPERLUSO := V_DSOPERLUSO || ' (' || TRIM(TO_CHAR(V_QTCAIXASFILIAL, '999999')) || ' caixa(s))';
                  V_DSOPERLINP := V_DSOPERLUSO || ' da unidade ' || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO;
               ELSE
                  IF CUR_CLIENTE.IDIDIOMA = 'ING' THEN
                     SELECT 'Apurao do valor mensal de ' || DECODE (CUR_LINP.IDTIPLINPC, 'M', 'manuteno', 'locao') INTO V_DSOPERLUSO FROM DUAL;
                     V_DSOPERLUSO := V_DSOPERLUSO || ' (' || TRIM(TO_CHAR(V_QTCAIXASFILIAL, '999999')) || ' caixa(s))';
                     V_DSOPERLINP := V_DSOPERLUSO || ' da unidade ' || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO;
                  ELSE
                     SELECT 'Apurao do valor mensal de '|| DECODE (CUR_LINP.IDTIPLINPC, 'M', 'manuteno', 'locao') INTO V_DSOPERLUSO FROM DUAL;
                     V_DSOPERLUSO := V_DSOPERLUSO || ' (' || TRIM(TO_CHAR(V_QTCAIXASFILIAL, '999999')) || ' caixa(s))';
                     V_DSOPERLINP := V_DSOPERLUSO || ' da unidade ' || CUR_UNIDADE.CDLOCALUSO || ' ' || CUR_UNIDADE.NMFANTLUSO;
                  END IF;
               END IF;
            END IF;

            IF CUR_LINP.IDTIPLINPC IN ('M', 'L') THEN
               IF V_LINPLUSO_VALIDA THEN
                  INSERT INTO LOGLINP
                    (CDCLIENTE, CDOPERADOR, CDLINPCLIE,
                     DTOPERLINP, DSOPERLINP, IDOPERLINP)
                  VALUES
                    (CUR_LINP.CDCLIENTE, P_CDOPERAPUR, CUR_LINP.CDLINPCLIE,
                     V_DTHRAPURACAO, V_DSOPERLINP, DECODE(CUR_LINP.IDTIPLINPC, 'M', 'APURACAO_MANUTENCAO', 'APURACAO_LOCACAO'));

                  INSERT INTO LOGLUSO
                    (CDCLIENTE, NRLICENCAUSO, CDOPERADOR, CDLINPCLIE,
                     DTOPERLUSO, DSOPERLUSO, TXARQUIVOPERS, IDOPERLUSO)
                  VALUES
                    (CUR_LINP.CDCLIENTE, CUR_UNIDADE.NRLICENCAUSO, P_CDOPERAPUR, CUR_LINP.CDLINPCLIE,
                     V_DTHRAPURACAO, V_DSOPERLUSO, NULL, DECODE(CUR_LINP.IDTIPLINPC, 'M', 'APURACAO_MANUTENCAO', 'APURACAO_LOCACAO'));
               END IF;
            ELSE
               IF V_QTCAIXASFILIAL > 0 THEN
                  INSERT INTO LOGLINP
                    (CDCLIENTE, CDOPERADOR, CDLINPCLIE,
                     DTOPERLINP, DSOPERLINP, IDOPERLINP)
                  VALUES
                    (CUR_LINP.CDCLIENTE, P_CDOPERAPUR, CUR_LINP.CDLINPCLIE,
                     V_DTHRAPURACAO, V_DSOPERLINP, DECODE(CUR_LINP.IDTIPLINPC, 'P', 'APURACAO_MANUTENCAO', 'APURACAO_LOCACAO'));

                  INSERT INTO LOGLUSO
                    (CDCLIENTE, NRLICENCAUSO, CDOPERADOR, CDLINPCLIE,
                     DTOPERLUSO, DSOPERLUSO, TXARQUIVOPERS, IDOPERLUSO)
                  VALUES
                    (CUR_LINP.CDCLIENTE, CUR_UNIDADE.NRLICENCAUSO, P_CDOPERAPUR, CUR_LINP.CDLINPCLIE,
                     V_DTHRAPURACAO, V_DSOPERLUSO, NULL, DECODE(CUR_LINP.IDTIPLINPC, 'P', 'APURACAO_MANUTENCAO', 'APURACAO_LOCACAO'));
               END IF;
            END IF;
         END LOOP; -- CURSOR DE UNIDADES POR LINHA DE PRODUTO DO CLIENTE

         -- ##############################################################################################
         -- APURA A DIFERENA ENTRE A QUANTIDADE TOTAL (MNIMA DE LICENAS OU DE LICENAS VLIDAS)
         -- EM RELAO  QUANTIDADE DE LICENAS J APURADAS ANTERIORMENTE (FILIAIS APURADAS SEPARADAMENTE)
         -- ##############################################################################################

         V_QTLICENCAS_A_FATURAR := V_QTLICENCAS_VALIDAS;
         IF V_QTLICENCAS_VALIDAS < CUR_LINP.QTMINIMAFATLINP THEN -- SE A QUANTIDADE DE LICENAS VLIDAS FOR MENOR QUE A
            V_QTLICENCAS_A_FATURAR := CUR_LINP.QTMINIMAFATLINP;  -- QUANTIDADE MNIMA CONTRATUAL DEVE FATURAR A QUANTIDADE MNIMA
         END IF;

         IF V_QTLICENCAS_JA_FATURADAS < V_QTLICENCAS_A_FATURAR THEN
            -- SE A QUANTIDADE DE LICENAS J FATURADAS ANTERIORMENTE FOR MENOR QUE A QUANTIDADE A SER FATURADA (VLIDAS OU MNIMA),
            -- FATURA A DIFERENA ENTRE A QUANTIDADE A SER FATURADA E A QUANTIDADE J FATURADA

            IF V_APURAR_UNIDADES_AGRUPADAS = 'S' THEN
               UPDATE APURFATUFAS
                  SET VRTOTAFATFAS = ROUND((V_QTLICENCAS_A_FATURAR - V_QTLICENCAS_JA_FATURADAS) * CUR_LINP.VRMENSALIDALINP, 2)
                WHERE NRAPURFATFAS = V_NRAPURFATFAS_AGRUPADA;
            ELSE
               NOVO_CODIGO('APURFATUFAS', V_NRAPURFATFAS);
               V_NRAPURFATFAS := SUBSTR(V_NRAPURFATFAS, 11, 10);

               INSERT INTO APURFATUFAS
                 (CDFILIAL, CDCLIENTE, NRAPURFATFAS,
                  DTAPURFATFAS, VRTOTAFATFAS, DSEMISSPRONF,
                  CDOPERAPUR, CDEMPRESA,CDPRODUTO,
                  IDAPURFATFAS, IDSTAPFATFAS, IDCOMIFATFAS,
                  DSBOLEFATFAS, DTMESAPURFAS, CDLINPCLIE,
                  CDGRUPONFAPUR, NRLICENCAUSO, IDFORMFATURAMEN)
               VALUES
                 (V_CDFILIAL, CUR_LINP.CDCLIENTE, V_NRAPURFATFAS,
                  V_DTAPURACAO, ROUND((V_QTLICENCAS_A_FATURAR - V_QTLICENCAS_JA_FATURADAS) * CUR_LINP.VRMENSALIDALINP, 2), V_DSFATURCLIE,
                  P_CDOPERAPUR, V_CDEMPRESA, V_CDPRODUTO,
                  '6', 'P', 'N',
                  DECODE(V_DESCLINHAPROD, 'S', V_DSBOLETCLIE || V_SISTEMAS, V_DSBOLETCLIE),
                  V_DTMES_APURACAO, CUR_LINP.CDLINPCLIE,
                  CUR_LINP.CDGRUPONFLINPC, NULL, DECODE(V_CLIENTE_IDFATURAPFILI, 'S', 'S', 'N'));

               IF V_GRAVOU_CONTRATOLINPAPU = 'N' THEN
                  INSERT INTO CONTRATOLINPAPU
                    (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, NMLINPCLIEAPUR,
                     NRUSUALINPAPUR, IDMPROCLINPAPUR, IDLPROCLINPAPUR, IDTIPLINPCAPUR,
                     QTMINIMAFATAPUR, VRLICENCUSOAPUR, VRMENSALIDAAPUR)
                  VALUES
                    (CUR_LINP.CDCLIENTE, CUR_LINP.CDLINPCLIE, V_DTMES_APURACAO, CUR_LINP.NMLINPCLIE,
                     CUR_LINP.NRUSUALINPC, CUR_LINP.IDMPROCLINPC, CUR_LINP.IDLPROCLINPC, CUR_LINP.IDTIPLINPC,
                     CUR_LINP.QTMINIMAFATLINP, CUR_LINP.VRLICENCUSOLINP, CUR_LINP.VRMENSALIDALINP);

                  INSERT INTO SISTCONTLINPAPU
                    (CDCLIENTE, CDLINPCLIE, DTMESAPURLINP, CDSISTEMA)
                  SELECT SC.CDCLIENTE, SC.CDLINPCLIE, V_DTMES_APURACAO, SC.CDSISTEMA
                    FROM SISTCONTLINP SC
                   WHERE SC.CDCLIENTE = CUR_LINP.CDCLIENTE
                     AND SC.CDLINPCLIE = CUR_LINP.CDLINPCLIE;
               END IF;
            END IF;
         END IF;
         COMMIT;
      END LOOP; -- CURSOR DE LINHAS DE PRODUTO DO CLIENTE
   END LOOP; -- CURSOR DE CLIENTES
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('FATURA_MANULOCA','5.05.012'); END;
/
