
// FILE: assets/js/Util/Utilities.js
function Utilities(ScreenService, templateManager, MessageService, ZhPreferences, ZhFilterPreferences, Query, ZHPromise,requestDataSourceEngine) {
    var self = this;

    this.SCRIPT_EXECUTION_SUCCESS = 'SCRIPT_SUCCESS';
    this.SCRIPT_EXECUTION_FAIL    = 'SCRIPT_FAIL';
    this.LOG_COMMAND_SEPARATOR    = '\n----------------------------------------------------------------------------------------------------------------------\n\n';

    this.setFilterInit = function (widget) {
        widget.mtAppliedFilter = true;
    };

    this.isAppliedFilter = function(widget){
        return !!widget.mtAppliedFilter;
    };

    this.getCurrentDateWithTime = function () {
        return moment().format('DD/MM/YYYY HH:mm:ss');
    };


    this.genericDeleteEvent = function (widget){
        return widget.dataSource.remove([widget.currentRow]).then(function(data){
            return ScreenService.closeSwipe().then(function(){
                return widget.reload();
            });
        });
    };

    this.getCurrentDateInMilliseconds = function(){
        return new Date().getTime();
    };

    this.getCurrentFormattedDateForAttachment = function(){
        return moment().format('YYYY-MM-DDTHH:MM:SSZ');
    };

    this.transliterateString = function(text) {
        var transliterationTable = {
            "á" : "a",  "Á" : "A",  "à" : "a",  "À" : "A",  "a" : "a",
            "A" : "A",  "â" : "a",  "Â" : "A",  "å" : "a",  "Å" : "A",
            "ã" : "a",  "Ã" : "A",  "ä" : "ae", "Ä" : "AE", "æ" : "ae",
            "Æ" : "AE", "c" : "c",  "C" : "C",  "ç" : "c",  "Ç" : "C",
            "d" : "d",  "D" : "D",  "Ð" : "Dh", "ð" : "dh", "é" : "e",
            "É" : "E",  "è" : "e",  "È" : "E",  "e" : "e",  "E" : "E",
            "ê" : "e",  "Ê" : "E",  "ë" : "e",  "Ë" : "E",  "ƒ" : "F",
            "g" : "g",  "G" : "G",  "h" : "h",  "H" : "H",  "í" : "i",
            "Í" : "I",  "ì" : "i",  "Ì" : "I",  "î" : "i",  "Î" : "I",
            "ï" : "i",  "Ï" : "I",  "i" : "i",  "I" : "I",  "j" : "j",
            "J" : "J",  "k" : "k",  "K" : "K",  "l" : "l",  "L" : "L",
            "n" : "n",  "N" : "N",  "ñ" : "n",  "Ñ" : "N",  "ó" : "o",
            "Ó" : "O",  "ò" : "o",  "Ò" : "O",  "ô" : "o",  "Ô" : "O",
            "o" : "o",  "O" : "O",  "õ" : "o",  "Õ" : "O",  "ø" : "oe",
            "Ø" : "OE", "ö" : "oe", "Ö" : "OE", "r" : "r",  "R" : "R",
            "s" : "s",  "S" : "S",  "š" : "s",  "Š" : "S",  "ß" : "SS",
            "t" : "t",  "T" : "T",  "ú" : "u",  "Ú" : "U",  "ù" : "u",
            "Ù" : "U",  "u" : "u",  "U" : "U",  "û" : "u",  "Û" : "U",
            "ü" : "ue", "Ü" : "UE", "w" : "w",  "W" : "W",  "ý" : "y",
            "Ý" : "Y",  "y" : "y",  "Y" : "Y",  "ÿ" : "y",  "Ÿ" : "Y",
            "z" : "z",  "Z" : "Z",  "ž" : "z",  "Ž" : "Z",  "þ" : "th",
            "Þ" : "Th", "µ" : "u"
        };
        var newText = "";
        if(text){
            for(var i = 0; i < text.length; i++){
                newText += transliterationTable[text[i]] || text[i];
            }
        }
        return newText;
    };

    this.b64Encode = function(str) {
        // first we use encodeURIComponent to get percent-encoded UTF-8,
        // then we convert the percent encodings into raw bytes which
        // can be fed into btoa.
        return btoa(encodeURIComponent(str).replace(/%([0-9A-F]{2})/g,
                function toSolidBytes(match, p1) {
                    return String.fromCharCode('0x' + p1);
                })
        );
    };

    this.byteLength = function(str) {
        var s = str.length;
        for (var i=str.length-1; i>=0; i--) {
            var code = str.charCodeAt(i);
            if (code > 0x7f && code <= 0x7ff) s++;
            else if (code > 0x7ff && code <= 0xffff) s+=2;
            if (code >= 0xDC00 && code <= 0xDFFF) i--;
        }
        return s;
    };

    this.showErrorMessage = function(error) {
        var message = error;
        if(typeof error === 'object') {
            if(error.message) {
                message = error.message;
            } else {
                message = error.data.error;
            }
        }
        ScreenService.showMessage(message, 'error', 'error');
    };

    /*fix bug SelectAutocomplete */
    this.initializeFieldSelectAutocomplete = function (widgetName, field) {
        var widget = templateManager.container.getWidget(widgetName);
        widget.getField(field).initializeField();
    };

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };


    this.showMessageErrorOfBackEnd = function(response) {
        return MessageService.getMessageByCode('UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode(messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    return ScreenService.showMessage(message, 'ERROR');
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.applyDefaultFilterOnEnter = function(widget, isApplyFilter) {
        var time = 500;
        return setTimeout(function() {
            self.applyDefaultFilter(widget, isApplyFilter);
        }, time);
    };

    this.setDefaultFilterOnEnter = function(widget, isApplyFilter) {
        self.applyDefaultFilterOnEnter(widget, isApplyFilter);
    };

    this.applyDefaultFilter = function(widget, isApplyFilter){
        if (widget.inListMode()) {
            if (widget.mtAppliedFilter) {
                widget.reload();
            } else if (widget.currentRowDefaultFilter) {
                self.startFilter(widget, widget.currentRowDefaultFilter, isApplyFilter);
            } else {
                self.startFilter(widget, widget.currentRow, isApplyFilter);
            }
        }
    };

    this.showMessageBackEndTrue = function(response) {
        return self.showMessageErrorOfBackEnd(response);
    };

    this.startFilter = function(widget, currentRow, isApplyOriginalFilter){
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };
                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if(defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else if(isApplyOriginalFilter) {
                ZhFilterPreferences.applyOriginalFilter(widget);
            } else {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        });
    };

    this.b64Decode = function(str) {
        // Going backwards: from bytestream, to percent-encoding, to original string.
        try {
            return decodeURIComponent(atob(str).split('').map(function (c) {
                return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
            }).join(''));
        } catch (e) {
            throw e;
        }
    };

    this.b64Encode = function(str) {
        // first we use encodeURIComponent to get percent-encoded UTF-8,
        // then we convert the percent encodings into raw bytes which
        // can be fed into btoa.
        return btoa(encodeURIComponent(str).replace(/%([0-9A-F]{2})/g,
                function toSolidBytes(match, p1) {
                    return String.fromCharCode('0x' + p1);
                })
        );
    };

    
    this.doSyncMT = function(functions) {
        return functions.reduce(function(lastPromise, nextFunction) {
            return lastPromise.then(function() {
                return nextFunction();
            });
        }, ZHPromise.when());
    };

    this.doSyncMtFeatParametersForScript = function(functions, currentRow, scripts, connType, logField, defer) {
        return functions.reduce(function(lastPromise, nextFunction, index) {
            return lastPromise.then(function() {
                return nextFunction(currentRow, scripts[index], connType, logField, defer);
            });
        }, ZHPromise.when());
    };
    
    this.mergeDataSetPagnation = function (field, filterMerge, valueFieldIndex, notFilter) {
        if(filterMerge){
            var filter = [];
            if(field.dataSourceFilter && !notFilter){
                filter = filter.concat(field.dataSourceFilter,filterMerge);
            }else{
                filter = filterMerge;
            }
            return requestDataSourceEngine.getDataSource(field.dataSource.name, filter).then(function (response) {
                var fieldValues = response.dataset[field.dataSource.name.split('/')[1]];
                fieldValues.forEach(function (fieldvalue) {
                    var validate = field.dataSource.data.filter(function (data) {
                        return data[valueFieldIndex] == fieldvalue[valueFieldIndex];
                    });
                    if (!validate.length) {
                        field.dataSource.data.push(fieldvalue);
                    }
                });
                templateManager.updateTemplate();
            });
        }
    };
    
    this.bugPaginationOnSelectMultiplyOnModeView = function(field){
        var valueFieldIndex = field.valueField;
        var filter = [
            {
                "operator": "IN",
                "name": valueFieldIndex,
                "value": field.value() || [0]
            }
        ];
        return this.mergeDataSetPagnation(field, filter, valueFieldIndex);
    };
    
    this.setValueInSelectField = function(field,value){
        field.widget.currentRow[field.outData[field.valueField]] = value;
        if(field.hasPagination !== false){
            this.bugPaginationOnSelectOnModeView(field,value).then(function(){
                self.setOutData(field, value);
            });
        }else {
            this.setOutData(field, value);
        }
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Utilities', Utilities);
});

// FILE: assets/js/Util/WebSocketClient.js
function WebSocketClient(GitQueue, ZHPromise) {

    this.connectInServer = function () {
        GitQueue.connect(wsServer);
    };

    this.loginInServer = function (id) {
        return GitQueue.loginBrowser(id);
    };

    this.createFile = function (path, fileName, fileContents, room) {
        var defer = ZHPromise.defer();
        var reject = function (error) {
            defer.reject(error);
        };
        var resolve = function () {
            defer.resolve();
        };
        (room || GitQueue).createPath(path)
            .error(reject)
            .then(function () {
                (room || GitQueue).createFile(path, fileName, fileContents)
                    .error(reject)
                    .then(resolve);
            });

        return defer.promise;
    };

    this.wget = function (path, url, fileName, room) {
        var defer = ZHPromise.defer();
        var reject = function (error) {
            defer.reject(error);
        };
        var resolve = function () {
            defer.resolve();
        };

        (room || GitQueue).wget(path, url, fileName)
            .error(reject)
            .then(resolve);

        return defer.promise;
    };

    this.move = function (src, path, room) {
        var defer = ZHPromise.defer();
        var reject = function (error) {
            defer.reject(error);
        };
        var resolve = function () {
            defer.resolve();
        };
        (room || GitQueue).move(src, path)
            .error(reject)
            .then(resolve);


        return defer.promise;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('WebSocketClient', WebSocketClient);
});



// FILE: assets/js/Service/Objects.js
function ObjectsService(requestDataSourceEngine, Utilities) {


    this.getObjectsBody = function(objectName, type, typeAbbreviation){
        var filter = [
            {
                "operator": "=",
                "name": "OBJECT_NAME",
                "value": objectName
            },
            {
                "operator": "=",
                "name": "OBJECT_TYPE",
                "value": type
            },
            {
                "operator": "=",
                "name": "TYPE_ABBREVIATION",
                "value": typeAbbreviation
            }
        ];
        return requestDataSourceEngine.getDataSource("/objectsBody" ,filter).then(function(response){
            return response.dataset.objectBody;
        });
    };

    this.loadTableObservations = function(tableName){
        var filter = [
            {
                "operator": "=",
                "name": "TABLE",
                "value": tableName
            }
        ];
        return requestDataSourceEngine.getDataSource("/tableObservations" ,filter).then(function(response){
            return response.dataset.findTableObservations[0];
        });
    };
    this.loadTableColumnObservations = function(columnName, tableName){
        var filter = [
            {
                "operator": "=",
                "name": "TABLE",
                "value": tableName
            },
            {
                "operator": "=",
                "name": "COLUMN",
                "value": columnName
            }
        ];
        return requestDataSourceEngine.getDataSource("/tableColumnObservations" ,filter).then(function(response){
            return response.dataset.findTableColumnObservations[0];
        });
    };

    this.saveTableObservation = function(widget){

        var dataset = [ {
            changed  : true,
            "row"    : widget.currentRow
        } ];

        return requestDataSourceEngine.setDataSource("/tableObservations" ,dataset).then(function(response){
            return response.dataset.tableObservations[0];
        });
    };

    this.saveTableColumnObservation = function(widget){

        var dataset = [ {
            changed  : true,
            "row"    : widget.currentRow
        } ];

        return requestDataSourceEngine.setDataSource("/tableColumnObservations" ,dataset).then(function(response){
            return response.dataset.tableColumnObservations[0];
        });
    };

    this.saveObjectsObservation = function(widget, type, objectName){

        var dataset = [ {
            changed  : true,
            "row"    : widget.currentRow,
            "type"   : type,
            "object_name" : objectName
        } ];

        return requestDataSourceEngine.setDataSource("/objectsObservation" ,dataset).then(function(response){
            return response.dataset.objectObservations;
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ObjectsService', ObjectsService);
});

// FILE: assets/js/Service/Structure.js
function StructureService(WebSocketClient, requestDataSourceEngine, Utilities, ScreenService, MessageService, ZHPromise, localStorageEngine) {

    var self = this;
    var DEFAULT_FILE_NAME = 'DesignEstrutura';
    var executedScriptsCount, totalScripts;
    var errorScripts = [];
    var successScripts = [];
    var scriptTargets = [];
    var hasDefaultValue = true;
    var valitedScript = true;
    var hasInvalidMultipleScripts = false;
    var uploadedScripts = [];
    var scriptArray = [];
    var scriptName = '';
    var objectName = '';
    var originalScript = '';
    var scriptIdsArray = [];
    var scriptDeferReturn = [];
    var isFromError = false;
    //Casos em que adicionara como nome 'versionados'
    var scriptsNameCases = ['VIEW', 'TRIGGER', 'TYPE', 'FUNCTION', 'PROCEDURE', 'PACKAGE'];
    //Casos para serem ignorados pela validação
    var exceptionCases = ['TEMPORARY', 'GLOBAL', 'FORCE'];

    var validateCreateScriptExceptions = function(currentScript){
        var constraint = [];
        var value = [];
        var crt = currentScript.slice(currentScript.indexOf("CREATE"), currentScript.indexOf("(")).split(" ");
        if(currentScript.indexOf("CONSTRAINT") >= 0){
            var allConstraints = currentScript.split("CONSTRAINT");
            for(var j in allConstraints){
                if(j > 0){
                   constraint.push(currentScript.split("CONSTRAINT")[j].split(" ")[1]);
                }
            }
        }
        if(currentScript.indexOf("CREATE OR REPLACE") >= 0){
            value = [{
                "Name": crt[4],
                "Action": crt[0] + crt[1] + crt[2],
                "Origin": crt[3],
                "Constraint": constraint
            }];
        }else{
            var i = 0;
            if(currentScript.indexOf("UNIQUE") >= currentScript.indexOf("CREATE") && currentScript.indexOf("UNIQUE") <= currentScript.indexOf("INDEX") ){
                i = 1;
            }
            value = [{
                "Name": crt[2 + i],
                "Action": crt[0],
                "Origin": crt[1 + i],
                "Constraint": constraint
            }];
        }
        scriptTargets.push(value);
    };

    var validateAlterScriptExceptions = function(currentScript){
        var constraint = [];
        var value = [];
        exceptionCases.some(function (v) {
            if(currentScript.includes(v)){
                currentScript = currentScript.replace(v, '');
            }
        });
        
        if (currentScript.indexOf("ADD") > 0 && currentScript.indexOf("ADD") < 40) {
            var tableName = currentScript.substring(currentScript.indexOf("TABLE") + 5, currentScript.indexOf("ADD"));
            tableName = tableName.replace(/\s/g, '');
            var alterNames = currentScript.split("ADD");
            if (currentScript.indexOf("CONSTRAINT") >= 0) {
                var allConstraints = currentScript.split("CONSTRAINT");
                for (var i in allConstraints) {
                    if (i > 0) {
                        constraint.push(currentScript.split("CONSTRAINT")[i].split(" ")[1]);
                    }
                }
            }
            if(alterNames){
                for(var n = 1; n < alterNames.length; n++){
                    var alterName = alterNames[n].split(" ")[1];
                    alterName = alterName.replace(/\s/g, '');

                     value = [{
                        "Name": alterName,
                        "Action": "ADD",
                        "Origin": "ALTER TABLE",
                        "Table": tableName,
                        "Constraint": constraint
                    }];
                    scriptTargets.push(value);
                }
            }else {


                 value = [{
                    "Name": " ERROR LOADING FIELD",
                    "Action": "ADD",
                    "Origin": "ALTER TABLE",
                    "Table": tableName,
                    "Constraint": constraint
                }];
                scriptTargets.push(value);
            }

        }
    };

    var validateNotNullScriptExceptions = function(currentScript){
        if (currentScript.indexOf("ADD ") > 0){
            var notNulls = currentScript.split("NOT NULL");
            for(var d in notNulls) {
                if( notNulls[d].lastIndexOf("DEFAULT",  notNulls[d].length) < 0 && notNulls[d].lastIndexOf(" IS ",  notNulls[d].length) < 0 ){
                    if(d >= 0 && notNulls[d] !== ""){
                        hasDefaultValue = false;
                    }
                } else {
                    hasDefaultValue = true;
                    return 0;
                }
            }
        }
    };

    var validateScriptExceptions = function(currentScript, isUploaded){
        hasDefaultValue = true;
        currentScript = currentScript.toUpperCase();
        var currentScriptToValidate = currentScript.replace(/(\r\n|\n|\r|\\n)/gm, ' ');
        if(currentScript.indexOf("CREATE") >= 0){
            validateCreateScriptExceptions(currentScriptToValidate);
            if(isUploaded && (currentScript.indexOf("INDEX") < 0 || currentScript.indexOf("INDEX") > 40)){
                uploadedScripts.objectCriation = uploadedScripts.objectCriation ? (uploadedScripts.objectCriation + "\n / \n" + currentScript) : currentScript;
            }else if (isUploaded && currentScript.indexOf("INDEX") > 0){
                uploadedScripts.structureUpdate = uploadedScripts.structureUpdate ? (uploadedScripts.structureUpdate + "\n / \n" + currentScript) : currentScript;
            }
        }else {
            if(isUploaded && currentScript.indexOf("PACKAGE") > 0){
                uploadedScripts.packages = uploadedScripts.packages ? (uploadedScripts.packages + "\n / \n" + currentScript) : currentScript;
            }else if(isUploaded){
                uploadedScripts.structureUpdate = uploadedScripts.structureUpdate ? (uploadedScripts.structureUpdate + "\n / \n" + currentScript) : currentScript;
            }
        }
        if(currentScript.indexOf("ALTER TABLE") >= 0) {
            validateAlterScriptExceptions(currentScriptToValidate);
        }
        if(currentScript.indexOf("NOT NULL") > 0){
            validateNotNullScriptExceptions(currentScriptToValidate);
        }
    };


    this.validateScriptNameIsNotSame = function(scripts, nmObjeto){
        objectName = '';
        scriptName = '';
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        scriptsNameCases.some(function (v) {
            if(scripts.includes(v)){
                scriptName = v;
            }
        });
        

        if(scriptName){
            var scriptsArray = getScriptsToExecute(scripts.split('\n'));
            if(scriptsArray.length > 1){
                reject();
            }
            
            if(nmObjeto){
                currentScript = scripts.replace('{', ' ');
                var crt = currentScript.slice(currentScript.indexOf(scriptName), currentScript.indexOf("(")).split(" ");
                objectName = crt[1];
                if(objectName != nmObjeto){
                    resolve(true);
                }else {
                    resolve(false);
                }
                
            }else{
                resolve(false);
            }
            
           
        }else{
            resolve(false);
        }

        return  defer.promise;
       
    };


     this.validateScriptName = function(scripts, nmObjeto, Id, connDb){
        objectName = '';
        scriptName = '';
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        scriptsNameCases.some(function (v) {
            if(scripts.includes(v)){
                scriptName = v;
            }
        });
       

        if(scriptName){
            var scriptsArray = getScriptsToExecute(scripts.split('\n'));
            if(scriptsArray.length > 1){
                reject();
            }
            if(nmObjeto){
                objectName = nmObjeto;
            }else{
                currentScript = scripts.replace('{', ' ');
                var crt = currentScript.slice(currentScript.indexOf(scriptName), currentScript.indexOf("(")).split(" ");
                objectName = crt[1];
            }
            
            getScriptNameIsInUse(objectName, Id, connDb).then(function(isNameInUse) {
                if(isNameInUse.length > 0){
                    resolve(true);
                }else{
                    resolve(false);
                }
            }).catch(reject);
            
           
        }else{
            resolve(false);
        }

        return  defer.promise;
    };

    var getScriptNameIsInUse = function(name, Id, connDb) {
        var dataset = [ {
            "operator": "=",
            "name": "NAME",
            "value": name
        },
        {
            "operator": "=",
            "name": "ID",
            "value": Id
        },
        {
            "operator": "=",
            "name": "DB",
            "value": connDb
        }  ];
        return requestDataSourceEngine.getDataSource('/getScriptNameIsInUse', dataset).then(function(response) {
            return response.dataset.ScriptNameIsInUse;
        });
    };

    this.validateScriptNameInOtherDbs = function(dbIds, objectName, scripts) {
        scriptName = '';
        var name = '';
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        scriptsNameCases.some(function (v) {
            if(scripts.includes(v)){
                scriptName = v;
            }
        });
        

        if(scriptName){
            currentScript = scripts.replace('{', ' ');
            name = currentScript.slice(currentScript.indexOf(scriptName), currentScript.indexOf("(")).split(" ")[1];
        }else if (objectName){
            name = objectName;
        }
        //
        if(name){
            var dataset = [ {
                "operator": "=",
                "name": "NAME",
                "value": name
            },
            {
                "operator": "=",
                "name": "DB",
                "value": dbIds
            }  ];
            requestDataSourceEngine.getDataSource('/validateScriptNameInOtherDbs', dataset).then(function(response) {
                resolve(response.dataset.ScriptNameIsInUse);
            });
        }else {
            resolve([]);
        }
        return defer.promise;
        
    };

    var validateScriptBeforeParse = function(scripts, currentRow, logField, logInitMessage, isUploaded, scriptId){
        var messageCode = "LOG_CANCEL_PARSE";
        if(isUploaded){
          messageCode = "LOG_INIT_SCRIPT_UPLOAD";
        }

        return MessageService.getMessageByCode(messageCode).then(function(messageCode) {
            valitedScript = true;
            for (var i in scripts){
                validateScriptExceptions(scripts[i], isUploaded);
                var validateDuplicated = hasDuplicatedArray(scriptTargets);
                var scriptObj = {
                    SCRIPT : scripts[i]
                };

                if(isUploaded) {
                    currentRow.LOG =  messageCode;
                }

                ScreenService.restoreDefaultLoadingMessage();

                if((validateDuplicated[0] || hasDefaultValue === false) && !isUploaded) {
                    var logCustomMessage = ScreenService.i18n(logInitMessage, scriptObj);

                        if( i < 1){
                            if(!currentRow.LOG){
                                currentRow.LOG =  messageCode + '\n' + logCustomMessage;
                            }else {
                                currentRow.LOG +=  messageCode + '\n' + logCustomMessage;
                            }
                        } else if(i > 0){
                            currentRow.LOG += Utilities.LOG_COMMAND_SEPARATOR + '\n' + logCustomMessage;
                        } else {
                            currentRow.LOG += '\n' + logCustomMessage;
                        }

                    var scriptStatus = [
                        {
                            "SCRIPT" : scripts[i],
                            "STATUS": "E",
                            "ID" : scriptId
                        }
                    ];
                    scriptArray.push(scriptStatus);
                    valitedScript = false;
                    currentRow.IDRESULTSCRIPT = "PE";
                    currentRow.INVALID_SCRIPT = true;
                    var messageParam = "";
                    var execMessage = "";
                    if(validateDuplicated[0]){
                        execMessage = 'LOG_ERROR_SCRIPT_EXECUTION_REP_NAME';
                        var lastDuplicated = scriptTargets.pop()[0];
                        if(lastDuplicated.Constraint.length > 0){
                            messageParam = {
                                NAME : lastDuplicated.Constraint
                            };
                        } else{
                            messageParam = {
                                NAME : lastDuplicated.Name
                            };
                        }
                    } else if(hasDefaultValue === false){
                        execMessage = 'LOG_ERROR_SCRIPT_NOT_NULL';
                    }
                    incrementLog(currentRow, null, logField, execMessage, messageParam);
                }
            }
        });
    };

    var saveScriptInPromisse = function(scripts, currentRow, connType, connDb, errorScripts, defer, widget){
        logValue = currentRow.LOG;
        var reject = function(error) {
            defer.reject(error);
        };
        self.getFullLog(currentRow.DSDIRLOG).then(function(response){
        logValue = response;
            currentRow.SCRIPT = originalScript;
            currentRow['SCRIPT_' + connDb] = originalScript;
            if(currentRow.ID){
                currentRow.ID = currentRow['ID_' + connDb];
            }
            if(!currentRow['IDRESULTSCRIPT_' + connDb]){
                currentRow.IDRESULTSCRIPT = null;

            }else{
                currentRow.IDRESULTSCRIPT = currentRow['IDRESULTSCRIPT_' + connDb];
            }
            self.saveScript(currentRow, connType, response, currentRow.REPOSITORIO_ID, connDb).then(function(response) {
                    if(!localStorageEngine.getLocalVar('FristScriptId')){
                        localStorageEngine.setLocalVar('FristScriptId', response.ID);
                    }
                    if(connType !== "exec"){
                        scriptIdsArray.push(response.ID); 
                    }
                    MessageService.getMessageByCode('WAIT').then(function(messageCode) {
                    ScreenService.changeLoadingMessage(messageCode);
                    //widget.currentRow = response;
                    var dbId = widget.currentRow.DB_ID;
                    Object.assign(widget.currentRow, response);
                    widget.currentRow.DB_ID = dbId;
                    widget.currentRow['SCRIPT_' + connDb] =  response.SCRIPT;
                    widget.currentRow['IDRESULTSCRIPT_' + connDb] =  response.IDRESULTSCRIPT;
                    widget.currentRow['ID_' + connDb] = response.ID;
                    widget.currentRow['DSDIRLOG_' + connDb] = response.DSDIRLOG;
                    widget.currentRow['NRSCRIPT_' + connDb] = response.NRSCRIPT;
                    if(errorScripts.length > 0) {
                        defer.resolve(Utilities.SCRIPT_EXECUTION_FAIL);
                    } else {
                        defer.resolve(widget.currentRow);
                   }

                  
                });
            }).catch(function() {
                reject();
            });
        }).catch(function() {
            reject();
        });
    };


    var executeParseScript = function(scripts, totalScripts, currentRow, logField, index, defer, connType , scriptId, connDb, widget){
        var reject = function(error) {
            defer.reject(error);
        };
        var resolve = function(success) {
            defer.resolve(success);
        };
        if(index == totalScripts) {
            if(isFromError){
                resolve(scriptArray);
            } else{
                saveScriptInPromisse(scripts, currentRow, connType, connDb, errorScripts, defer, widget);
            }
        } else {
            var currentScript = scripts[index].trim();
            if(Util.isEmptyOrBlank(currentScript)) {
                executeParseScript(scripts, totalScripts, currentRow, logField, ++index, defer, connType , scriptId, connDb, widget);
            } else{
                MessageService.getMessageByCode('LOG_INIT_SCRIPT_EXECUTION').then(function(logInitMessage) {

                    if(connType == "parse" && index < 1){
                        var initLog = currentRow.LOG;
                        validateScriptBeforeParse(scripts, currentRow, logField, logInitMessage, false, scriptId).then(function(){
                            if(valitedScript){
                                if(!currentRow.LOG){
                                    currentRow.LOG = initLog;
                                }
                                self.executeScripts(currentScript, currentRow, logField, connType, logInitMessage, scripts, totalScripts, index, defer, scriptId, connDb, widget);

                            }
                        }).catch(reject);
                    }else {
                        if(valitedScript){
                            self.executeScripts(currentScript, currentRow, logField, connType, logInitMessage, scripts, totalScripts, index, defer, scriptId, connDb, widget);
                        }
                    }
                    //scared
                });
            }
        }
    };

    this.relateScripts = function(){

        var dataset = [ {
            changed : true,
            "STRUCTURES_IDS": scriptIdsArray,

        } ];
        return requestDataSourceEngine.setDataSource('/relateScripts', dataset).then(function(){
            scriptIdsArray = [];
        }).catch(function () {
            scriptIdsArray = [];
        });

    };

     this.executeScripts = function(currentScript, currentRow, logField, connType, logInitMessage, scripts, totalScripts, index, defer, scriptId, connDb, widget){            
        var reject = function(error) {
            defer.reject(error);
        };
        self.executeScript(currentScript, connType, connDb).then(function(scriptResponse){

            var scriptObj = {
                SCRIPT : currentScript
            };
            logInitMessage = ScreenService.i18n(logInitMessage, scriptObj);
            currentRow.LOG += '\n' + logInitMessage;

            var messageParam = null;
            var scriptResult = scriptResponse.dataset.scriptResult[0];
            var scriptResultId = scriptResponse.dataset.scriptResult.result;
            var execMessage = scriptResultId == "S" ? 'LOG_SUCCESS_SCRIPT_EXECUTION' : 'LOG_ERROR_SCRIPT_EXECUTION' ;

            ScreenService.restoreDefaultLoadingMessage();

            var scriptStatus = [
                {
                    "SCRIPT" : currentScript,
                    "STATUS": scriptResultId,
                    "ID" : scriptId
                }
            ];
            scriptArray.push(scriptStatus);
            incrementLog(currentRow, scriptResult, logField, execMessage, messageParam);
            if(scriptResultId == "E" && !errorScripts.includes(currentScript)) {
                if (!currentRow['SCRIPT_ERR_' + connDb]) {
                    currentRow.SCRIPT_ERR = currentScript;
                    currentRow['SCRIPT_ERR_' + connDb] = currentScript;
                } else {
                    currentRow['SCRIPT_ERR_' + connDb] += '\n/\n' + currentScript;
                }
                errorScripts.push(currentScript);
            } else if(scriptResultId == "S" && !successScripts.includes(currentScript)){
                if(!currentRow['SCRIPT_SUC_' + connDb]){
                    currentRow['SCRIPT_SUC_' + connDb] = currentScript;
                } else {
                    currentRow.SCRIPT_SUC += '\n/\n' + currentScript;
                    currentRow['SCRIPT_SUC_' + connDb] += '\n/\n' + currentScript;
                }
                successScripts.push(currentScript);
            }


            executeParseScript(scripts, totalScripts, currentRow, logField, ++index, defer, connType, scriptId, connDb, widget);
        }).catch(function (tableResponse) {
            var msg = ScreenService.i18n("Erro desconhecido");
            if(tableResponse && tableResponse.data && tableResponse.data.error) {
                msg = tableResponse.data.error;
            }
            incrementLog(currentRow,  msg, logField, "LOG_ERROR_SCRIPT_EXECUTION");
            errorScripts.push(currentScript);
            reject();
        });

    };
    var findAnyElementWithingArray = function(source, target){
        var result = source.filter(function(item){
            return target.indexOf(item) > -1;
        });
        return (result.length > 0);
    };

    var hasDuplicatedArray = function(a) {
        var k = [];
        for(var i in a){
            for(var j in a){
                if(a[i].length && a[j].length){
                    var matchedConstraint = findAnyElementWithingArray(a[i][0].Constraint, a[j][0].Constraint);
                    if(i!=j && a[i][0].Constraint !== "" && matchedConstraint){
                        if(k.indexOf(i) < 0){
                            k.push(i);
                        }
                    }
                }

                if(i!=j && JSON.stringify(a[i]) == JSON.stringify(a[j])){
                    if(k.indexOf(i) < 0){
                        k.push(i);
                    }
                }
            }
        }
        return k;
    };

    var buildErrorScriptForParse = function(currentRow, script, connType, logField, widget){
        isFromError = true;
        var defer = ZHPromise.defer();
        var reject = function(error) {
            defer.reject(error);
        };
        MessageService.getMessageByCode('LOG_INIT_SCRIPTS').then(function(messageInit) {
            originalScript = script.SCRIPT;
            var scripts      = removeCharsInScriptComments(script.SCRIPT);
            var scriptId = script.ID;
            var connDb = script.DB_ID;
            scripts = getScriptsToExecute(scripts.split('\n'));
            executedScriptsCount = 0;
            totalScripts = scripts.length;
            messageInit = ScreenService.i18n(messageInit);
            if(!currentRow.LOG){
                currentRow.LOG = messageInit;
            }
            executeParseScript(scripts, scripts.length, currentRow, logField, 0, defer, connType, scriptId, connDb, widget);
   
        }).catch(function() {
            reject();
        });
        return defer.promise;
    };

    this.executeScriptsFromError = function(widget, scriptsFromError, connType, logField, dbId){
        var currentRow = widget.currentRow;
        var defer = ZHPromise.defer();
        var resolves = function(success) {
            defer.resolve(success);
        };
        var rejects = function(error) {
            defer.reject(error);
        };
        successScripts = []; errorScripts = [];
        scriptTargets = [];
        scriptArray = [];
        valitedScript = true;
        currentRow.LOG = "";
        var functions = [];
        scriptsFromError.forEach(function(script) {
            functions.push(buildErrorScriptForParse.bind(StructureService, currentRow, script, connType, logField, widget));
        });                                             
        Utilities.doSyncMT(functions).then(function(){
            self.getFullLog(widget.currentRow.DSDIRLOG).then(function(response) {
                self.updateScript(widget.currentRow, connType, response, currentRow.REPOSITORIO_ID, dbId).then(function(response) {
                    var dbIds = widget.currentRow.DB_ID;
                    Object.assign(widget.currentRow, response);
                    widget.currentRow.DB_ID = dbIds;
                    widget.currentRow['SCRIPT_' + dbId] =  response.SCRIPT;
                    widget.currentRow['IDRESULTSCRIPT_' + dbId] =  response.IDRESULTSCRIPT;
                    widget.currentRow['ID_' + dbId] = response.ID;
                    widget.currentRow['DSDIRLOG_' + dbId] = response.DSDIRLOG;
                    widget.currentRow['NRSCRIPT_' + dbId] = response.NRSCRIPT;
                    resolves();
                }).catch(function() {
                    rejects();
                });
            }).catch(function() {
                rejects();
            });
        }).catch(function() {
            rejects();
        });
        return defer.promise;
    };

    this.executeScriptInPromisse = function(widget, scripts, connType, logField, connDb, defer) {
        currentRow = widget.currentRow;
        successScripts = []; errorScripts = [];
        scriptTargets = [];
        valitedScript = true;
        originalScript = scripts;
        scripts       = removeCharsInScriptComments(scripts);
        scripts = getScriptsToExecute(scripts.split('\n'));
        executedScriptsCount = 0;
        totalScripts = scripts.length;
        isFromError = false;
        MessageService.getMessageByCode('LOG_INIT_SCRIPTS').then(function(messageInit) {
            messageInit = ScreenService.i18n(messageInit);
            currentRow.LOG = messageInit;
            currentRow.SCRIPT_ERR = "";
            currentRow.SCRIPT_SUC = "";
            scriptArray = [];
            executeParseScript(scripts, scripts.length, currentRow, logField, 0, defer, connType, null, connDb, widget);
            
        });
    };

    var getScriptsToExecute = function(tempScripts) {
        var scripts = [];
        var idx = 0;
        tempScripts.forEach(function(tempScript) {
            var trimTempScript = tempScript.trim();
            if(trimTempScript !== '') {
                if (trimTempScript[0] == '/' && trimTempScript[1] != '*') {
                    idx++;
                } else {
                    scripts[idx] = scripts[idx] ? scripts[idx] + '\n' + tempScript : tempScript;
                }
            }
        });
        //reset keys
        return scripts.filter(function() { return true; });
    };

    var removeCharsInScriptComments = function (currentScript) {
        var idx         = 0;
        var script      = '';
        var tempScript  = '';

        while(idx < currentScript.length) {
            if(currentScript[idx] == "'") {
                tempScript = getTempScript(currentScript, idx + 1, "'");
                script += "'" + tempScript.SCRIPT + "'";
                idx = idx + tempScript.LAST_IDX + 2;
            } else if(currentScript[idx] == "-" && currentScript[idx + 1] == "-") {
                tempScript = getTempScript(currentScript, idx + 2, "\n");
                script += "--" + replaceQuotationMarks(tempScript.SCRIPT) + "\n";
                idx = idx + tempScript.LAST_IDX + 3;
            } else if(currentScript[idx] == "/" && currentScript[idx + 1] == "*") {
                tempScript = getTempScript(currentScript, idx + 2, "*/");
                script += "/*" + replaceQuotationMarks(tempScript.SCRIPT) + "*/";
                idx = idx + tempScript.LAST_IDX + 4;
            } else {
                script += currentScript[idx];
                idx++;
            }
        }
        return script;
    };
    var getIndecesOf = function(searchStr, str){
        var searchStrLen = searchStr.length;
        if (searchStrLen === 0) {
            return [];
        }
        var startIndex = 0, index, indices = [];
        while ((index = str.indexOf(searchStr, startIndex)) > -1) {
            indices.push(index);
            startIndex = index + searchStrLen;
        }
        return indices;
    };

    this.validateScriptsComments = function (currentScript) {

        //True -> validação valida, false -> validação Invalida
        if(currentScript.match(/\/\*/g) && currentScript.match(/\*\//g)){
            var opencomment = currentScript.match(/\/\*/g).length;
            var closecomment = currentScript.match(/\*\//g).length;
            if(opencomment == closecomment){
                var openCommentIndices = getIndecesOf("/*", currentScript);
                var closeCommentIndices = getIndecesOf("*/", currentScript);
                var isValid =  true;
                for(i=0; i <= (openCommentIndices.length);i++){
                    if(closeCommentIndices[i] < openCommentIndices[i] || closeCommentIndices[i] > openCommentIndices[i+1] ){
                        isValid = false;
                    }
                }
                return isValid;
            }else{
                return false;
            }
        }else {
            return !(currentScript.match(/\*\//g) || currentScript.match(/\/\*/g));
        }

    };


    var getTempScript = function(currentScript, idx, endDelimiter) {
        var endIndexOff = currentScript.substr(idx).indexOf(endDelimiter);
        return {
            'SCRIPT'  : currentScript.substr(idx).substr(0, endIndexOff),
            'LAST_IDX': endIndexOff
        };
    };

    var replaceQuotationMarks = function(script) {
        return script.replace(/'/g, '').replace(/"/g, '');
    };
    var getLogfiles = function(row, type, oldLog, repository, connDb) {
        if(!row.LOG && !oldLog) {
            return ZHPromise.when({
                "value" : false
            });
        }
        return getServerLogFileContents(row, row.OPERATT, repository, row.LOG, type, connDb).then(function(log){
            var fullLog = "";
            if(oldLog){
                fullLog = oldLog +  log;
            } else {
                fullLog =  log;

            }
            var arr = [];
            arr.push(buildLogDocument(row,fullLog));
            return arr;
        });

    };

    this.downloadScriptLog = function(id, description, logDir) {
        description = description ? description.replace(/#/g, '').replace(/&/g, '').trim() : "";
        var routeStr           = '/downloadScriptLog';
        var requestTypeStr     = '?requestType=DataSet';
        var datasetFileNameStr = '&dataset[0][FILE_NAME]=' + getLogFileName(id, description);
        var datasetFileDirStr  = '&dataset[0][FILE_DIR]=' + logDir;
        window.open(window.serviceUrl + routeStr + requestTypeStr + datasetFileNameStr + datasetFileDirStr);
    };

    this.downloadScript = function(id) {
        var routeStr       = '/downloadScript';
        var requestTypeStr = '?requestType=DataSet';
        var datasetIdStr   = '&dataset[0][ID]=' + id;
        window.open(window.serviceUrl + routeStr + requestTypeStr + datasetIdStr);
    };

    var getLogFileName = function(id, description) {
        return (description || DEFAULT_FILE_NAME) + '_' + id + '.log';
    };

    this.getFullLog = function(dirLog){
        var filter = [
            {
                "operator": "=",
                "name": "DIRLOG",
                "value": dirLog
            }
        ];
        return requestDataSourceEngine.getDataSource('/getFullLog', filter).then(function(response){
            return response.dataset.logFileContents[0];
        });
    };

    this.getFullLogs = function(row){
        var filter = [
            {
                "operator": "=",
                "name": "ROW",
                "value": row
            }
        ];
        return requestDataSourceEngine.getDataSource('/getFullLogs', filter).then(function(response){
            return response.dataset.logFileContents;
        });
    };


    var getLogHeader = function(repository, dbId){
        var filter = [
            {
                "name" : "ID",
                "operator": "=",
                "value": repository
            },
            {
                "name" : "DB_ID",
                "operator": "=",
                "value": dbId
            }
        ];
        return requestDataSourceEngine.getDataSource("/getUserAndRepository" ,filter).then(function(response){
            return response.dataset.logHeader;
        });
    };

    var getServerLogFileContents = function(row, user, repository, currentLogContents, typeLog, dbId) {
        return getLogHeader(repository, dbId).then(function(logHeader){
            var log = currentLogContents ? currentLogContents : "";
            var logFileContents = '\n-----> Design Estrutura: ' + row.ID + '-' + row.DSDESESTRUTURA;
            logFileContents += ' | ' + ScreenService.i18n('Usuário') + ': ' + logHeader[1];
            logFileContents += ' | ' + ScreenService.i18n('Repositório') + ': ' + logHeader[0];
            logFileContents += ' | ' + ScreenService.i18n('Data') + ': ' + Utilities.getCurrentDateWithTime();
            logFileContents += ' | ' + ScreenService.i18n('Banco') + ': ' + logHeader[2];
            logFileContents += ' | ' + typeLog + ' <-----\n\n' + log + '\n\n';
            return logFileContents;
        });
    };

    var buildLogDocument = function(row,rowLog){
        var log = Utilities.transliterateString(rowLog).replace(/[\u00A0-\u2666]/g, '');
        return  {
            name              : (row.DSDESESTRUTURA || DEFAULT_FILE_NAME) + '.log',
            b64File           : 'data:text/plain;base64,' + Utilities.b64Encode(log),
            lastModified      : Utilities.getCurrentDateInMilliseconds(),
            lastModifiedDate  : Utilities.getCurrentFormattedDateForAttachment(),
            size              : Utilities.byteLength(rowLog),
            type              : 'text/plain',
            webkitRelativePath: ''
        };
    };

    this.uploadScripts = function(scripts, logField, currentRow, repository){
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };

        scripts         = removeCharsInScriptComments(scripts);
        var scriptarray = getScriptsToExecute(scripts.split('\n'));
        executedScriptsCount = 0;
        totalScripts = scriptarray.length;
        uploadedScripts = [];

        divideScriptsUpload(scriptarray, logField, currentRow, repository).then(function() {
            var script = [] ;
            currentRow.SCRIPT_SUC = "";
            currentRow.DSDESESTRUTURA = currentRow.DOCUMENT[0].name;
            currentRow.SCRIPT_ERR = "";
            currentRow.IDATIVO = "S";
            currentRow.IDRESULTSCRIPT = "PD";
            currentRow.REPOSITORIO_ID = repository;

            script.push(uploadedScripts.packages);
            script.push(uploadedScripts.structureUpdate);
            if(uploadedScripts.objectCriation){
                uploadedScripts.objectCriation = getScriptsToExecute(uploadedScripts.objectCriation.split('\n'));
                uploadedScripts.objectCriation.forEach(function (obj) {
                    script.push(obj);
                });
            }

            currentRow.SCRIPT = script;

            saveUploadedScript(currentRow, repository).then( function() {
                resolve();
            }).catch(reject);
        }).catch(reject);

        return defer.promise;
    };

    var saveUploadedScript = function(currentRow, repository){
        currentRow.REPOSITORIO_ID = repository;

        var dataset = [ {
            changed : true,
            "row"   : currentRow
        } ];
        return requestDataSourceEngine.setDataSource('/structureUpload', dataset).then(function(response) {
            return response.dataset.savedScripts[0];
        });

    };

    var divideScriptsUpload = function (scripts, logField, currentRow){
        return MessageService.getMessageByCode('LOG_INIT_SCRIPT_EXECUTION').then(function(logInitMessage) {
            return validateScriptBeforeParse(scripts, currentRow , logField, logInitMessage, true);
        });
    };

    var buildScriptSql = function(nrscript,rowScript){
        var script = Utilities.transliterateString(rowScript).replace(/[\u00A0-\u2666]/g, '');
        return  {
            name              : nrscript + '.sql',
            b64File           : 'data:text/plain;base64,' + Utilities.b64Encode(script),
            lastModified      : Utilities.getCurrentDateInMilliseconds(),
            lastModifiedDate  : Utilities.getCurrentFormattedDateForAttachment(),
            size              : Utilities.byteLength(rowScript),
            type              : 'text/plain',
            webkitRelativePath: ''
        };
    };

    this.saveScript = function(currentRow, type, oldLog, repository, connDb) {
        if(type == 'parse'){
            objectName = '';
        }
        if(currentRow.NMOBJETO){
            objectName = currentRow.NMOBJETO;
        }
        //var firstScript = localStorageEngine.getLocalVar('FristScriptId');
        currentRow.REPOSITORIO_ID = repository;
        return getLogfiles(currentRow, type, oldLog, repository, connDb).then(function(logFiles) {
            var dataset = [ {
                changed : true,
                "row"   : currentRow,
                "LOG"   : logFiles,
                "SCRIPTARRAY" : scriptArray,
                "NMOBJETO": objectName,
                "DB_ID": connDb
            } ];
            return requestDataSourceEngine.setDataSource('/structure', dataset).then(function(response) {
                return response.dataset.savedScripts[0];
            });
        });
    };

    this.updateScript = function(currentRow, type, oldLog, repository, connDb) {
        currentRow.ID = currentRow['ID_' + connDb];
        currentRow.REPOSITORIO_ID = repository;
        if(!currentRow['IDRESULTSCRIPT_' + connDb]){
            currentRow.IDRESULTSCRIPT = null;

        }else{
            currentRow.IDRESULTSCRIPT = currentRow['IDRESULTSCRIPT_' + connDb];
        }
        return getLogfiles(currentRow, type, oldLog, repository, connDb).then(function(logFiles) {
            var dataset = [ {
                changed : true,
                "row"   : currentRow,
                "LOG"   : logFiles,
                "SCRIPTARRAY" : scriptArray
            } ];
            return requestDataSourceEngine.setDataSource('/structureUpdate', dataset).then(function(response) {
                return response.dataset.savedScripts[0];
            });
        });
    };

    this.inactivateScript = function(nmObjetoRow, connDb) {
        if(nmObjetoRow){
            objectName = nmObjetoRow;
        }
            var dataset = [ {
                changed : true,
                "NMOBJETO"   : objectName,
                "CONN_DB" : connDb
            } ];
            return requestDataSourceEngine.setDataSource('/inactivateScript', dataset).then(function(response) {
                return response.dataset.inactivateScript;
            });
    };

    this.inactivateScriptStructure = function(id) {
            var dataset = [ {
                changed : true,
                "ID"   : id
            } ];
            return requestDataSourceEngine.setDataSource('/inactivateScriptStructure', dataset).then(function(response) {
                return response.dataset.inactivateScriptStructure;
            });
    };

    this.getErrorsScripts = function(currentRow) {
            var dataset = [ {
                "operator": "=",
                "name": "ID",
                "value": currentRow.ID
            } ];
            return requestDataSourceEngine.getDataSource('/getScriptsWithError', dataset).then(function(response) {
                return response.dataset.Scripts;
            });
    };

    this.getRelatedScripts = function($nrEstrutRel) {
        var filter = [
            {
                "operator": "=",
                "name": "NRESTRUTREL",
                "value": $nrEstrutRel
            }
        ];
        return requestDataSourceEngine.getDataSource("/findRelatedScript", filter).then(function (response) {
            return response.dataset.findRelatedScript;
        });
};

    this.executeScript = function(script, connType, connDb) {    
        var dataset = [ {
            changed : true,
            "SCRIPT": script,
            "connectionType": connType,
            "connectionDataBase": connDb

        } ];
        return requestDataSourceEngine.setDataSource('/structureScript', dataset);
    };

    var incrementLog = function (structureUpdateRow, outputScript, logField, messageCode, messageParam) {
        var commandSeparator = "";
        if(messageCode == "LOG_SUCCESS_SCRIPT_EXECUTION" || messageCode == "LOG_ERROR_SCRIPT_EXECUTION") {
            commandSeparator = Utilities.LOG_COMMAND_SEPARATOR;
        }

        return MessageService.getMessageByCode(messageCode).then(function(messageCode){
            if(messageParam){
                messageCode = ScreenService.i18n(messageCode, messageParam);
            } else {
                messageCode = ScreenService.i18n(messageCode);
            }

            if(outputScript) {
                structureUpdateRow.LOG += outputScript + '\n\n';
            }
            structureUpdateRow.LOG += messageCode + commandSeparator;

            var loadMessage = ScreenService.i18n("Executando Scripts");
            return ScreenService.changeLoadingMessage(loadMessage + ": " + (++executedScriptsCount) + '/' + totalScripts);
        });

    };

    this.validateWebSocketDirectory = function(idRep){

        var filter = [
            {
                "operator": "=",
                "name": "IDREP",
                "value": idRep
            }
        ];
        return requestDataSourceEngine.getDataSource('/validateWebSocketDirectory', filter).then(function(response){
            return response.dataset.validateWebSocketDirectory[0];
        });
    };


    this.removeTempFile = function(url) {
        var filter = [ {
            "operator": "=",
            "name": "Dir",
            "value": url
        } ];
        requestDataSourceEngine.getDataSource('/removeScriptOnTempFile', filter);
    };

    this.downloadByWebSocket = function(clientEmail, dir, structure, dbId) {
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };

        var dataset = [ {
            changed  : true,
            "row"    : structure,
            "SCRIPT" : [buildScriptSql(structure['NRSCRIPT_' + dbId], structure['SCRIPT_' + dbId])]
        } ];

        requestDataSourceEngine.setDataSource('/scriptOnTempFile', dataset).then(function(response) {
            var url = response.dataset.scriptOnTempFile[0];
            var pathFileServer = response.dataset.scriptOnTempFile[1];
            var directorySeparator = "";

            var room = "";
            var fileName = structure['NRSCRIPT_' + dbId] + ".sql";
            if(dir.includes("/")){
                directorySeparator = "/";
            }else {
                directorySeparator = "\\";
            }
            var src  = dir + directorySeparator + fileName + ".tar.gz";
            var dest = dir + directorySeparator + fileName;

            WebSocketClient.loginInServer(clientEmail).then(function (rooms) {
                room = rooms;
                return WebSocketClient.wget(dir, url, fileName, room);
            }).then(function () {
                return WebSocketClient.move(src, dest, room);
            }).then(function() {
                resolve(pathFileServer);
            }).catch(reject);
        }).catch(reject);

        return defer.promise;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('StructureService', StructureService);
});

// FILE: assets/js/Controller/Functions.js
function FunctionsController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getFunctionsWidget = function () {
        return templateManager.container.getWidget('FUNCTIONS');
    };
    // --------------------------- END -------------------------------//

    this.functionsWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getFunctionsWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.functionsBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.functionBasicEditAction = function(){
        var widget = getFunctionsWidget();
        localStorageEngine.setLocalVar('functionObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.functionBasicCancelAction = function(){
        getFunctionsWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('functionObservation');
        getFunctionsWidget().view();
    };

    this.functionsWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getFunctionsWidget());
    };

    this.functionsWidgetAfterMoveRow = function (){
        var widget = getFunctionsWidget();
        var name = widget.currentRow.OBJECT_NAME;
        var type = 'FUNCTION';
        ObjectsService.getObjectsBody(name, type, 'F').then(function(objectBody){
            widget.currentRow.OBJECT_BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.functionActionSaveObservation = function (){
        var widget = getFunctionsWidget();
        ObjectsService.saveObjectsObservation(widget, 'F', widget.currentRow.OBJECT_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('FunctionsController', FunctionsController);
});

// FILE: assets/js/Controller/Index.js
function IndexController(ScreenService, localStorageEngine,  templateManager, Utilities, ObjectsService) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getIndexWidget = function () {
        return templateManager.container.getWidget('INDEX');
    };
    var getIndexColumnsWidget = function () {
        return templateManager.container.getWidget('INDEX_COLUMNS');
    };
    // --------------------------- END -------------------------------//

    this.indexWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getIndexWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.indexActionSaveObservation = function (){
        var widget = getIndexWidget();
        ObjectsService.saveObjectsObservation(widget, 'I', widget.currentRow.INDEX_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

    this.indexWidgetAfterMoveRow = function (){
        var widget = getIndexWidget();
        var name = widget.currentRow.INDEX_NAME;
        ObjectsService.getObjectsBody(name, null, 'I').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            widget.currentRow.OBSERVATION = objectBody.OBSERVATION;

        });
    };

    this.columnWidgetAfterMoveRow = function (){
        var widget = getIndexColumnsWidget();
        ObjectsService.loadTableColumnObservations(widget.currentRow.INDEX_NAME, widget.currentRow.TABLE_NAME).then(function(response){
            widget.currentRow.OBSERVATION = response;
        });
    };

    this.indexBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.indexBasicEditAction = function(){
        var widget = getIndexWidget();
        localStorageEngine.setLocalVar('indexObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.indexBasicCancelAction = function(){
        getIndexWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('indexObservation');
        getIndexWidget().view();
    };

    this.indexColumnWidgetOnEnter = function(){
        getIndexColumnsWidget().reload();
    };

    this.indexWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getIndexWidget());
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('IndexController', IndexController);
});

// FILE: assets/js/Controller/Job.js
function JobController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getJobWidget = function () {
        return templateManager.container.getWidget('JOB');
    };
    // --------------------------- END -------------------------------//

    this.jobWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getJobWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.jobBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.jobWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getJobWidget());
    };

    this.jobBasicCancelAction = function (){
        getJobWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('jobObservation');
        getJobWidget().view();
    };

    this.jobBasicEditAction = function (){
        var widget = getJobWidget();
        localStorageEngine.setLocalVar('jobObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.jobWidgetAfterMoveRow = function (){
        var widget = getJobWidget();
        var name = widget.currentRow.JOB_NAME;
        var type = 'PROCOBJ';
        ObjectsService.getObjectsBody(name, type, 'J').then(function(objectBody){
            widget.currentRow.BODY= objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.jobActionSaveObservation = function (){
        var widget = getJobWidget();
        ObjectsService.saveObjectsObservation(widget, 'J', widget.currentRow.JOB_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('JobController', JobController);
});

// FILE: assets/js/Controller/Procedure.js
function ProcedureController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getProcedureWidget = function () {
        return templateManager.container.getWidget('PROCEDURE');
    };
    // --------------------------- END -------------------------------//

    this.procedureWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getProcedureWidget();
            //ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.procedureBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.procedureBasicEditAction = function(){
        var widget = getProcedureWidget();
        localStorageEngine.setLocalVar('procedureObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.procedureBasicCancelAction = function(){
        getProcedureWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('procedureObservation');
        getProcedureWidget().view();
    };

    this.procedureWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getProcedureWidget());
    };

    this.procedureWidgetAfterMoveRow = function (){
        var widget = getProcedureWidget();
        var name = widget.currentRow.OBJECT_NAME;
        var type = 'PROCEDURE';
        ObjectsService.getObjectsBody(name, type, 'P').then(function(objectBody){
            widget.currentRow.OBJECT_BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.procedureActionSaveObservation = function (){
        var widget = getProcedureWidget();
        ObjectsService.saveObjectsObservation(widget, 'P', widget.currentRow.OBJECT_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ProcedureController', ProcedureController);
});

// FILE: assets/js/Controller/Repositorio.js
function RepositorioController(ScreenService, localStorageEngine, MessageService, templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getRepositorioWidget = function () {
        return templateManager.container.getWidget('REPOSITORIO');
    };
    // --------------------------- END -------------------------------//

    this.repositorioWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getRepositorioWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.repositorioActionEventAdd = function() {
        getRepositorioWidget().newRow();
    };

    this.repositorioBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.repositorioBasicCancelAction = function(){
        var widget = getRepositorioWidget();
        if(widget.currentRow.__is_new){
            ScreenService.closeSwipe();
        } else {
            widget.view();
        }
        getRepositorioWidget().reload();
    };

    this.repositorioWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getRepositorioWidget());
    };

    this.repositorioActionEventEdit = function (){
        var widget = getRepositorioWidget();
        widget.edit();
    };

    this.repositorioActionEventDelete = function () {
        MessageService.confirmMessageByCode('ALE_CONFIRM_REMOVE_REGISTER', null,
            {
                type: 'QUESTION',
                success: function() {
                    Utilities.genericDeleteEvent(getRepositorioWidget()).then(function() {
                        MessageService.successNotificationByCode('SUC_DELETED');
                    }).catch(Utilities.showMessageErrorOfBackEnd);
                },
                fail: function () {}
            });
    };

    this.repositorioActionEventSave = function (){
        var widget = getRepositorioWidget();
        widget.saveRow().then(function(response){
            if(widget.currentRow.__is_new){
                ScreenService.closeSwipe();
            }else {
                widget.view();
            }
            widget.reload();
            MessageService.successNotificationByCode('SUC_CREATED');
        }).catch(Utilities.showMessageErrorOfBackEnd);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('RepositorioController', RepositorioController);
});

// FILE: assets/js/Controller/Sequence.js
function SequenceController(ScreenService, localStorageEngine,  templateManager, Utilities, ObjectsService) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getSequenceWidget = function () {
        return templateManager.container.getWidget('SEQUENCE');
    };
    // --------------------------- END -------------------------------//

    this.sequenceWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getSequenceWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.sequenceBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.sequenceBasicEditAction = function(){
        var widget = getSequenceWidget();
        localStorageEngine.setLocalVar('sequenceObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.sequenceBasicCancelAction = function(){
        getSequenceWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('sequenceObservation');
        getSequenceWidget().view();
    };

    this.sequenceWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getSequenceWidget());
    };

    this.sequenceWidgetAfterMoveRow = function (){
        var widget = getSequenceWidget();
        var name = widget.currentRow.SEQUENCE_NAME;
        ObjectsService.getObjectsBody(name, null, 'S').then(function(objectBody){
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.sequenceActionSaveObservation = function (){
        var widget = getSequenceWidget();
        ObjectsService.saveObjectsObservation(widget, 'S', widget.currentRow.SEQUENCE_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('SequenceController', SequenceController);
});

// FILE: assets/js/Controller/Structure.js
function StructureController(ZHPromise, WebSocketClient, ScreenService, StructureService, localStorageEngine, MessageService, templateManager, Utilities, ZhFilterPreferences, metaDataFactory, requestDataSourceEngine) {

    var self = this;
    var logValue = "";
    var tempScript = null;
    var isFromErr = null;
    var logErrEdit = false;
    var logExec = false;
    var scriptsFieldsNames = [];
    var scriptsFieldsDBs = [];

    // ------------------------ GET WIDGETS ------------------------ \\

    var getStructureWidget = function () {
        return templateManager.container.getWidget('ESTRUTURA');
    };
    var getStripeWidget = function () {
        return templateManager.container.getWidget('STRIPE_NAVIGATOR');
    };
    var getRepositoryWidget = function () {
        return templateManager.container.getWidget('REPOSITORY');
    };
    var getViewLogWidget = function () {
        return templateManager.container.getWidget('VIEWLOG');
    };

    var getWidgetScriptUpload = function() {
        return templateManager.container.getWidget('SCRIPT_UPLOAD');
    };

    var getWidgetScriptFix = function() {
        return templateManager.container.getWidget('SCRIPT_FIX');
    };
    // -------------------------- Fields --------------------------- \\

    var getScriptFromStructureWidget = function() {
        return getStructureWidget().getField('SCRIPT');
    };
    var getDescriptionFromStructureWidget = function() {
        return getStructureWidget().getField('DSDESESTRUTURA');
    };
    var getIdAtivoFromStructureWidget = function() {
        return getStructureWidget().getField('IDATIVO');
    };
    var getLogFromStructureWidget = function() {
        return getStructureWidget().getField('LOG');
    };
    var getLogFromLogWidget = function() {
        return getViewLogWidget().getField('LOG');
    };
    var getRepositoryIdFromRepositoryWidget = function() {
        return getRepositoryWidget().getField('REPOSITORIO_ID');
    };
    var getRepositoryIdFromEstruturaWidget = function() {
        return getStructureWidget().getField('REPOSITORIO_ID');
    };

    // --------------------------- ACTIONS ------------------------- \\

    var getActionParsefromStructureWidget = function () {
        return getStructureWidget().getAction('EXECSCRIPT');
    };
    var getActionBaixarLogfromStructureWidget = function () {
        return getStructureWidget().getAction('LOGDOWNLOAD');
    };
    var getActionBaixarScriptfromStructureWidget = function () {
        return getStructureWidget().getAction('SCRIPTDOWNLOAD');
    };
    var getActionErrEditStructureWidget = function () {
        return getStructureWidget().getAction('ERREDIT');
    };
    var getActionViewLogStructureWidget = function () {
        return getStructureWidget().getAction('VIEWLOG');
    };
    var getActionExecuteStructureWidget = function () {
        return getStructureWidget().getAction('EXECUTE');
    };
    var getActionEditStructureWidget = function () {
        return getStructureWidget().getAction('EDIT');
    };
    var getActionSaveDescStructureWidget = function () {
        return getStructureWidget().getAction('SAVEDESC');
    };
    var getActionInactivateScript = function () {
        return getStructureWidget().getAction('INACTIVATE');
    };
    var getActionErrEditViewLogWIdget = function () {
        return getViewLogWidget().getAction('ERREDIT');
    };
    var getActionExecViewLogWIdget = function () {
        return getViewLogWidget().getAction('EXEC');
    };
    
    
    var getActionRemoveStructureWidget = function () {
        return getStructureWidget().getAction('REMOVE');
    };
    var getFieldIssuesStructureWidget = function(){
        return getStructureWidget().getField('ISSUES');
    };
    var getFieldIssuesUplodStructureWidget = function(){
        return getWidgetScriptUpload().getField('ISSUES_UPLOAD');
    };

    // --------------------------- END ----------------------------- \\


    this.getScriptFromErrorField = function (projectId, widget, dbId) {
        return metaDataFactory.factoryField(this.getScriptFromError(projectId, dbId), widget);
    };
    // ---------------------- Dynamic Field ------------------- //
    
    this.getScriptFromError = function (scriptId, dbId) {
        return {
            "name": "SCRIPT_" + scriptId,
            "label": "Script",
            "readOnly": false,
            "fieldGroup": "DB_" + dbId,
            "template": "field/textarea.html",
            "help": "Atalhos :<br> <br> Ctrl-F: Pesquisar <br> Ctrl-Space: Autocomplete <br> Alt-G: Pesquisar Linha <br> F11: Tela Cheia",
            "maxWidth": "15%",
            "textRows": 10,
            "mtIsGeneration": true,
            "disableOrdering": true,
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12,
            "filterProperties": {
                "help": false,
                "class": 5,
                "sequenceInForm": 3,
                "isVisible": true,
                "template": "field/text-edit.html"
            }
        };
    };

    // -=======================================================-//

    this.structureCloseSwipe = function() {
        ScreenService.closePopup();
    };

    this.structureCloseFixErrorsPopup = function() {
        var widgetFix = getWidgetScriptFix();
        var fields = widgetFix.fields;
        widgetFix.fields = self.mtGenerationProjectIdFilter(fields);
        ScreenService.closePopup(0);
        getStructureWidget().activate();
    };

    this.structureActionChangeRep = function() {
        var repositoryWidget = getRepositoryWidget();
        ScreenService.openPopup(repositoryWidget);
    };

    this.structureWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            var nrRepository = localStorage.getItem('repositorio');
            ScreenService.hideLoader();
            var widget = getStructureWidget();
            widget.currentRow = {
                "IDRESULTSCRIPT": ['PD' , 'PS', 'PE', 'EE'],
                "IDATIVO": 'S'
            };
            ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.structureWidgetOnEnterOnce = function() {
        var repositoryWidget = getRepositoryWidget();
        var repository = localStorage.getItem('repositorio');
        if(!repository) {
            ScreenService.openPopup(repositoryWidget);
        } else {
            loadStripeRepositorio();
        }
    };

    var loadStripeRepositorio = function(){
        var stripeWidget = getStripeWidget();
        var repository = localStorage.getItem('repositorio');
        localStorageEngine.setLocalVar('repositorio', repository);
        stripeWidget.reload().then(function(){
            stripeWidget.moveToFirst();
            localStorageEngine.setLocalVar('nomeRepositorio', stripeWidget.currentRow.NMREPOSITORIO);
        });
    };

    this.structureActionSaveRepository = function() {
        var stripeWidget = getStripeWidget();
        var repositoryWidget = getRepositoryWidget();
        var widgetStructure = getStructureWidget();

        if(repositoryWidget.currentRow.REPOSITORIO_ID) {
            localStorage.setItem('repositorio', repositoryWidget.currentRow.REPOSITORIO_ID);
            var repository = localStorage.getItem('repositorio');
            localStorageEngine.setLocalVar('repositorio', repository);
        }
        loadStripeRepositorio();
        ScreenService.closePopup(0);
        widgetStructure.reload();

    };

    this.structureBasicBackAction = function(){
        validateActions();
        var widget = getStructureWidget();
        ScreenService.closeSwipe();
        widget.list();
    };

    this.structureViewLogBackAction = function(){
        logErrEdit = false;
        logExec = false;
        ScreenService.closePopup(0);
        getStructureWidget().activate();
        validateActions();
    };

    this.structureBasicCancelAction = function(){
        localStorageEngine.setLocalVar('CodeMirror', null);
        var widget = getStructureWidget();
        if(tempScript) {
            widget.currentRow.SCRIPT = tempScript;
            tempScript = null;
        }
        if(widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
            widget.reload();
        } else {
            widget.view();
        }

        isFromErr = false;
        validateActions();
    };

    this.structureWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getStructureWidget());
    };

    this.structureActionEventAdd = function () {
        var repositorioId = localStorage.getItem('repositorio');
        if(!repositorioId) {
            ScreenService.openPopup(getRepositoryWidget());
        } else {
            var widget = getStructureWidget();
            validateActions();
            widget.newRow();
            getRepositoryIdFromEstruturaWidget().reload();
            widget.currentRow.REPOSITORIO_ID = localStorage.getItem('repositorio');
            setCodeMirror(false);
            var issueField = getFieldIssuesStructureWidget();
            requestDataSourceEngine.getDataSource("/findIssuesRelatedToTask", []).then(function (response) {
                    if(response.dataset && response.dataset.findIssuesRelatedToTask){
                        var issueId = response.dataset.findIssuesRelatedToTask;
                        issueField.value(issueId);
                    }
                });
        }
    };
    
    this.structureActionDescSave = function(){
        var repository = localStorage.getItem('repositorio');
        var widget = getStructureWidget();
        var type = 'EDIT';
        widget.currentRow.TYPE_SAVE = type;
        var codeMirror = localStorageEngine.getLocalVar('CodeMirror');
        if(widget.isValid()){
            widget.currentRow.SCRIPT = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow.SCRIPT;
            StructureService.saveScript(widget.currentRow, type, null, repository).then(function(response){
            ScreenService.closePopup();
            widget.view();
            Object.assign(widget.currentRow, response);
            widget.currentRow = merged;
            }).catch(Utilities.showMessageErrorOfBackEnd);  
        } else {
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
        
    };

    var removeAllFieldsMtGeneration = function(widget){
        widget.fields = widget.fields.filter(function(field){
            if(!field.mtTempField){
                return field;
            }
            
        });
    };

    var removeAllFieldsGroupsMtGeneration = function(widget){
        widget.fieldGroups = widget.fieldGroups.filter(function(field){
            if(!field.mtTempField && field.name){
                return field;
            }
        });
    };

    this.getScriptNumberField = function (dbId, widget) {
        return metaDataFactory.factoryField(this.getScriptNumberObj(dbId), widget);
    };

    this.getScriptIdField = function (dbId, widget) {
        return metaDataFactory.factoryField(this.getScriptIdObj(dbId), widget);
    };

    this.getScriptResultField = function (dbId, widget) {
        return metaDataFactory.factoryField(this.getScriptResultObj(dbId), widget);
    };
    
    this.getScriptField = function (dbId, widget, readOnly) {
        if(!readOnly){
            readOnly = false;
        }
        return metaDataFactory.factoryField(this.getScriptObj(dbId, readOnly), widget);
    };

    this.getFieldGroup = function (dbId, name, widget) {
        return metaDataFactory.factoryField(this.getFieldGroupSelectObj(dbId, name), widget);
    };

    this.getLogField = function (dbId, name, widget) {
        return metaDataFactory.factoryField(this.getLogFieldSelectObj(dbId), widget);
    };

    this.getAuditFieldGroup = function (dbId, name, widget) {
        return metaDataFactory.factoryField(this.getAuditFieldGroupSelectObj(), widget);
    };

    this.getAuditFieldGroupSelectObj = function (dbId, name) {
        return {
            "name": "AUDIT",
            "label": "Auditoria",
            "collapsible": false,
            "mtTempField": true,
            "opened": true
        };
    };


    this.getFieldGroupSelectObj = function (dbId, name) {
        return {
            "name": "DB_" + dbId,
            "label": name,
            "mtTempField": true,
            "collapsible": false,
            "opened": true
        };
    };

    this.getScriptObj = function (dbId, readOnly) {
        return {
                "name": "SCRIPT_" + dbId,
                "label": "Script",
                "readOnly": readOnly,
                "template": "field/textarea.html",
                "help": "Atalhos :<br> <br> Ctrl-F: Pesquisar <br> Ctrl-Space: Autocomplete <br> Alt-G: Pesquisar Linha <br> F11: Tela Cheia",
                "maxWidth": "15%",
                "fieldGroup": "DB_" + dbId,
                "textRows": 15,
                "disableOrdering": true,
                "isVisible": true,
                "showOnList": true,
                "showOnView": true,
                "mtTempField": true,
                "sequenceInForm": 11,
                "class": 12
        };
    };

    this.getLogFieldSelectObj = function (dbId) {
        return {
                "name": "LOG_" + dbId,
                "label": "Log",
                "template": "field/textarea.html",
                "maxWidth": "20%",
                "textRows": 15,
                "mtTempField": true,
                "disableOrdering": true,
                "readOnly": true,
                "isVisible": true,
                "showOnList": false,
                "showOnView": true,
                "class": 12,
                "sequenceInForm": 12,
        };
    };

    this.getScriptResultObj = function (dbId) {
        return {
                "name": "IDRESULTSCRIPT_" + dbId,
                "label": "Resultado",
                "maxWidth": "15%",
                "readOnly": true,
                "fieldGroup": "DB_" + dbId,
                "template": "field/select-native.html",
                "valueField": "ID",
                "descriptionField": "NAME",
                "dataSource": {
                    "data": [
                        {
                            "ID": "PD",
                            "NAME": "Pendente"
                        },
                        {
                            "ID": "PE",
                            "NAME": "Parse com Erro"
                        },
                        {
                            "ID": "PS",
                            "NAME": "Parse com Sucesso"
                        },
                        {
                            "ID": "EE",
                            "NAME": "Executado com Erro"
                        },
                        {
                            "ID": "ES",
                            "NAME": "Executado com Sucesso"
                        }
                    ]
                },
                "disableOrdering": true,
                "isVisible": true,
                "showOnList": true,
                "showOnView": true,
                "mtTempField": true,
                "sequenceInForm": 10,
                "class": 4
        };
    };

    this.getScriptNumberObj = function (dbId) {
        return {
            "label": "Número Script",
            "name": "NRSCRIPT_" + dbId,
            "showOnList": false,
            "class": 4,
            "maxWidth": "10%",
            "fieldGroup": "DB_" + dbId,
            "readOnly": true,
            "sequenceInForm": 9,
            "mtTempField": true,
            "template": "field/text-edit.html"
        };
    };

    this.getScriptIdObj = function (dbId) {
        return {
            "label": "Id.",
            "name": "ID_" + dbId,
            "showOnList": true,
            "sequenceInForm": 1,
            "fieldGroup": "DB_" + dbId,
            "descriptionOrder": 1,
            "readOnly": true,
            "mtTempField": true,
            "class": 4,
            "template": "field/text-edit.html",
        };
    };

    this.structureDbOnChange = function(){
        var widget = getStructureWidget();
        removeAllFieldsMtGeneration(widget);
        removeAllFieldsGroupsMtGeneration(widget);
        var fields = widget.fields;
        var fieldGroups = widget.fieldGroups;
        widget.currentRow.DB_ID.forEach(function (dbId, index) {
                var name = widget.currentRow.DB_NAME[index];
                var fieldGroup = self.getFieldGroup(dbId, name, widget);
                var scriptNumberField = self.getScriptNumberField(dbId, widget);
                var scriptIdField = self.getScriptIdField(dbId, widget);
                var scriptField = self.getScriptField(dbId, widget);
                var scriptResultField = self.getScriptResultField(dbId, widget);
                fields.push(scriptIdField);
                fields.push(scriptNumberField);
                fields.push(scriptResultField);
                fields.push(scriptField);
                var scriptFieldName = "SCRIPT_" + dbId;
                setCodeMirror(false, scriptFieldName);
                fieldGroups.push(fieldGroup);
        });
        fieldGroups.push(self.getAuditFieldGroup());
        widget.fieldGroups = fieldGroups;
        widget.fields = fields;
    };

    this.structureActionParseScriptsWithErrors = function() {
        var widgetFix = getWidgetScriptFix();
        var widget = getStructureWidget();
        var logField   = getLogFromStructureWidget();
        widgetFix.currentRow = [];
        var singleScript = [];
        var type = "parse";
        scriptsFieldsDBs.forEach(function(dbId){
            singleScript[dbId] = [];
            var index = 0;

            if(widget.currentRow['IDRESULTSCRIPT_' + dbId] == "EE"){
                type =  "exec";
            }
            scriptsFieldsNames[dbId].forEach(function(script){
               
                singleScript[dbId][index] = {};
                var scriptName = script;
                var codeMirror = localStorageEngine.getLocalVar(scriptName);
                if(codeMirror){
                    singleScript[dbId][index].SCRIPT = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow[scriptName];
                    singleScript[dbId][index].ID = scriptName.substr(7);
                    singleScript[dbId][index].DB_ID = dbId;

                    index += 1;
                }
    
            });
        });
        var functions = [];
        widget.currentRow.DB_ID.forEach(function(dbId) {
            if(singleScript[dbId]){
                functions.push(executeScriptFromErrorController.bind(StructureController, widget, singleScript[dbId], type, logField, dbId));
            }
        });
        
        Utilities.doSyncMT(functions).then(function(response) {
            var widgetFix = getWidgetScriptFix();
            var fields = widgetFix.fields;
            widget.reload();
            widget.view();
            widgetFix.fields = self.mtGenerationProjectIdFilter(fields);
            showLogAfterSave();
        }).catch(function() {
            ScreenService.restoreDefaultLoadingMessage();
        });
    };

    var executeScriptFromErrorController = function(widget, singleScript, type, logField, dbId){
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        
        StructureService.executeScriptsFromError(widget, singleScript, type, logField, dbId).then(function(resp){
            ScreenService.restoreDefaultLoadingMessage();
            resolve();
        }).catch(function() {
            ScreenService.restoreDefaultLoadingMessage();
            reject();
        });
        return defer.promise;

    };

    var showLogAfterSave = function(resp, scriptIds){
        var widget = getStructureWidget();
        if (!widget.__is_new) {
            ScreenService.closePopup();
        }
        widget.currentRow.LOG = logValue;
        if (isFromErr) {
            isFromErr = false;
        }
        widget.view();
        var message = "";
        var result = "";
        type = "parse";
        var errorCount = 0;
        var execSucc = false;
        if(!Array.isArray(widget.currentRow.DB_ID)){
            widget.currentRow.DB_ID = [widget.currentRow.DB_ID];
        }
        widget.currentRow.DB_ID.forEach(function(dbId){
            
            if(widget.currentRow['IDRESULTSCRIPT_' + dbId] == "EE"){
                type =  "exec";
            }

            if (widget.currentRow['IDRESULTSCRIPT_' + dbId] == "PE" || widget.currentRow['IDRESULTSCRIPT_' + dbId] == "EE" || widget.currentRow['IDRESULTSCRIPT_' + dbId] == "PD") {
                errorCount += 1;
            } else if (widget.currentRow['IDRESULTSCRIPT_' + dbId] == "ES") {
                execSucc = true;
            }
            
        });
        

        if(errorCount > 0){
            logErrEdit = true;
            logExec = false;
            result = "ERROR";
            message = "ERR_SCRIPT_RESULT";
            MessageService.showMessageByCode(message, null, {type: result});
        }else{
            logErrEdit = false;
            result = "SUCCESS";
            message = "SUC_SCRIPT_RESULT";
            if (execSucc) {
                logExec = false;
                self.validateWebSocketDirectory(resp);
            } else {
                logExec = true;
                MessageService.showMessageByCode(message, null, {type: result});
            }
            
        }
        validateActions();
        self.structureActionEventViewLog();
        localStorageEngine.setLocalVar('CodeMirror', null);
    };

    this.structureActionExecuteScript = function(){
        var widget = getStructureWidget();  
        var dangerousStatement = false;
        if(widget.isValid() && widget.currentRow.ISSUES){
            widget.currentRow.DB_ID.forEach(function(dbId) {
            var name = "SCRIPT_" + dbId;
            codeMirror = localStorageEngine.getLocalVar(name);
            var scripts = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow['SCRIPT_' + dbId]; 
                
            if (scripts.indexOf("DROP") >= 0 || scripts.indexOf("TRUNCATE") >= 0) {
               dangerousStatement = true;
            } else {
                dangerousStatement = false;
            }
            });
            
            if(dangerousStatement){
                MessageService.showCustomDialogByCode('ALE_DANGEROUS_STATEMENT', null, {
                    type: 'CONFIRMATION',
                    actions: [{
                        "label": ScreenService.i18n("Prosseguir"),
                        "code": function () {
                            validateScriptObjectName("exec");
                        }
                    }, {
                        "label": ScreenService.i18n("Cancelar"),
                        "code": function () {
                            
                        }
                    }]
                });
            }else {
                validateScriptObjectName("exec");
            }
        } else{
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
    };

    this.structureViewInactivateAction = function(){
        var widget     = getStructureWidget();
        var objectName = widget.currentRow.NMOBJETO;
        MessageService.confirmMessageByCode('ALE_CONFIRM_INACTIVE_SCRIPT', null,
        {
            type: 'QUESTION',
            success: function() {
                
                StructureService.getRelatedScripts(widget.currentRow.ID).then(function(response){
                    Object.assign(widget.currentRow, response);
                    StructureService.inactivateScriptStructure(widget.currentRow.ID).then(function(){
                        MessageService.successNotificationByCode('SUC_INACTIVATED');
                        ScreenService.closePopup();
                        ScreenService.closeSwipe();
                    });
                });
                
            },
            fail: function () {}
        });
        
    };

    this.structureActionParse = function (){
        var widget = getStructureWidget();  
        var dangerousStatement = false;
        if(widget.isValid() && widget.currentRow.ISSUES){
            widget.currentRow.DB_ID.forEach(function(dbId) {
                var name = "SCRIPT_" + dbId;
                codeMirror = localStorageEngine.getLocalVar(name);
                var scripts = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow['SCRIPT_' + dbId]; 
                    
                if (scripts.indexOf("DROP") >= 0 || scripts.indexOf("TRUNCATE") >= 0) {
                dangerousStatement = true;
                } else {
                    dangerousStatement = false;
                }
            });
            
            if(dangerousStatement){
                MessageService.showCustomDialogByCode('ALE_DANGEROUS_STATEMENT', null, {
                    type: 'CONFIRMATION',
                    actions: [{
                        "label": ScreenService.i18n("Prosseguir"),
                        "code": function () {
                            validateScriptObjectName("parse");
                        }
                    }, {
                        "label": ScreenService.i18n("Cancelar"),
                        "code": function () {
                            
                        }
                    }]
                });
            }else {
                validateScriptObjectName("parse");
            }
        } else{
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
    };

    var addRemainingDbOnScreen = function (dbIds, defer) {
        var reject = function(error) {
            defer.reject(error);
        };
        var widget = getStructureWidget();
        dbIds.forEach(function (db){
            widget.currentRow.DB_ID.push(db.ID);
            widget.currentRow.DB_NAME.push(db.NAME);
        });
        widget.redraw();
        removeAllFieldsMtGeneration(widget);
        removeAllFieldsGroupsMtGeneration(widget);
        var fields = widget.fields;
        var fieldGroups = widget.fieldGroups;
        var auditFieldGroup = fieldGroups.pop();
        widget.currentRow.DB_ID.forEach(function (dbId, index) {
                var scriptName = "SCRIPT_" + dbId;
                var codeMirror = localStorageEngine.getLocalVar(scriptName);
                if(codeMirror){
                    widget.currentRow[scriptName] = codeMirror.doc.getValue();
                }
                var name = widget.currentRow.DB_NAME[index];
                var fieldGroup = self.getFieldGroup(dbId, name, widget);
                var scriptNumberField = self.getScriptNumberField(dbId, widget);
                var scriptIdField = self.getScriptIdField(dbId, widget);
                var scriptField = self.getScriptField(dbId, widget);
                var scriptResultField = self.getScriptResultField(dbId, widget);
                fields.push(scriptIdField);
                fields.push(scriptNumberField);
                fields.push(scriptResultField);
                fields.push(scriptField);
                var scriptFieldName = "SCRIPT_" + dbId;
                setCodeMirror(false, scriptFieldName);
                fieldGroups.push(fieldGroup);
        });
        fieldGroups.push(auditFieldGroup);
        widget.fieldGroups = fieldGroups;
        widget.fields = fields;
        reject();
    };

    var validateScriptObjectNamePromisse = function (connType, dbId){
        setCodeMirror(false);
        var widget     = getStructureWidget();
        var scripts = widget.currentRow['SCRIPT_' + dbId];
        if(connType == 'parse'){
            var name = "SCRIPT_" + dbId;
            codeMirror = localStorageEngine.getLocalVar(name);
            scripts = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow['SCRIPT_' + dbId]; 
            
        }
        var objectName = widget.currentRow.NMOBJETO;
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        //var connDb = dbId;

        var invalidComments = StructureService.validateScriptsComments(scripts);
           if(!invalidComments){
               MessageService.showMessageByCode('ERR_INVALID_COMMENT');
               return false;
           }else {
               if(connType == 'exec'){
                var isInUsePromisse = StructureService.validateScriptName(scripts, widget.currentRow.NMOBJETO, widget.currentRow.ID, dbId);
                isInUsePromisse.then(function(isInUse){
                   if(isInUse){
                        MessageService.confirmMessageByCode('ALE_CONFIRM_REMOVE_SAME_OBJECT_STRUCTURES', null,
                        {
                            type: 'QUESTION',
                            success: function() {
                                StructureService.inactivateScript(objectName, dbId).then(function(){
                                    resolve();
                                });
                            },
                            fail: function () {
                                reject();
                            }
                        });
                   }else {
                        resolve();
                   }
                }).catch(function() {
                    MessageService.showMessageByCode('ERR_MULTIPLE_SCRIPT');
                    reject();
                });
               }else{
                var itIsSame = StructureService.validateScriptNameIsNotSame(scripts, objectName);
                itIsSame.then(function(isNotSame){
                    if(isNotSame){
                        MessageService.confirmMessageByCode('ALE_CONFIRM_DIFFERENT_OBJECT_STRUCTURES', null,
                    {
                        type: 'QUESTION',
                        success: function() {
                            var otherDb = StructureService.validateScriptNameInOtherDbs(widget.currentRow.DB_ID, objectName, scripts);
                            otherDb.then(function(DbUsingObjectIds){
                                if(DbUsingObjectIds.length >= 1){
                                    MessageService.confirmMessageByCode('ALE_CONFIRM_OBJECT_DIFFERENT_DB', null,
                                {
                                    type: 'QUESTION',
                                    success: function() {
                                        addRemainingDbOnScreen(DbUsingObjectIds, defer);
                                    },
                                    fail: function () {
                                        reject();
                                    }
                                });
                                }else{
                                    resolve(); 
                                }     
                            }).catch(function() {
                                MessageService.showMessageByCode('ERR_VALIDACAO_BD');
                                reject();
                            });
                        },
                        fail: function () {
                            reject();
                        }
                    });
                    }else{
                        var otherDb = StructureService.validateScriptNameInOtherDbs(widget.currentRow.DB_ID, objectName, scripts);
                            otherDb.then(function(DbUsingObjectIds){
                                if(DbUsingObjectIds.length >= 1){
                                    MessageService.confirmMessageByCode('ALE_CONFIRM_OBJECT_DIFFERENT_DB', null,
                                {
                                    type: 'QUESTION',
                                    success: function() {
                                        addRemainingDbOnScreen(DbUsingObjectIds, defer);
                                    },
                                    fail: function () {
                                        reject();
                                    }
                                });
                                }else{
                                    resolve(); 
                                }     
                            }).catch(function() {
                                MessageService.showMessageByCode('ERR_VALIDACAO_BD');
                                reject();
                            });
                    }     
                }).catch(function() {
                    MessageService.showMessageByCode('ERR_MULTIPLE_SCRIPT');
                    reject();
                });

                    
               }
           }

        return defer.promise;

    };

    var validateScriptObjectName = function (connType){
        setCodeMirror(false);
        var widget     = getStructureWidget();
        var objectValidation = [];
        StructureService.scriptIdsArray = [];
        widget.currentRow.DB_ID.forEach(function(dbId) {
            objectValidation.push(validateScriptObjectNamePromisse.bind(StructureController, connType, dbId));
        });
        Utilities.doSyncMT(objectValidation).then(function(response) {
            var scriptExecFunction = [];
            widget.currentRow.DB_ID.forEach(function(dbId) {
                scriptExecFunction.push(execAndSaveControl.bind(StructureController, connType, dbId));
            });
            var scriptIds = [];
            Utilities.doSyncMT(scriptExecFunction).then(function(response) {
                widget.reload();

                if(connType == 'exec'){
                    showLogAfterSave(response, scriptIds);
                }else {
                    return StructureService.relateScripts().then(function() {
                        showLogAfterSave(response, []);
                    }).catch(function() {
                        ScreenService.restoreDefaultLoadingMessage();
                    });
                }
                
            });
            
        });
       

        

    };

    var execAndSaveControl = function(connType, dbId){
        var widget     = getStructureWidget();
        var scripts = widget.currentRow['SCRIPT_' + dbId];
        if(connType == 'parse' && !isFromErr){
            var name = "SCRIPT_" + dbId;
            codeMirror = localStorageEngine.getLocalVar(name);
            scripts = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow['SCRIPT_' + dbId]; 
            
        }
        var logField   = getLogFromStructureWidget();

        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        StructureService.executeScriptInPromisse(widget, scripts, connType, logField, dbId, defer);

        return defer.promise;
    };


    this.structureViewExecAction = function(){
        ScreenService.closePopup();
        logExec = false;
        self.structureActionExecuteScript();
    };

    this.mtGenerationProjectIdFilter = function(array){
        return array.filter(function (field) {
            return !field.mtIsGeneration;
        });
    };

    this.structureActionErrEdit = function (){
        var widget = getStructureWidget();
        scriptsFieldsNames = [];
        scriptsFieldsDBs = [];
        var widgetFix = getWidgetScriptFix();
        removeAllFieldsMtGeneration(widgetFix);
        removeAllFieldsGroupsMtGeneration(widgetFix);
        var fields = widgetFix.fields;
        var fieldGroups = widgetFix.fieldGroups;
        StructureService.getErrorsScripts(widget.currentRow).then(function(scriptIds){
            ScreenService.closePopup();
            scriptIds.forEach(function(scriptId){
                if(!scriptsFieldsNames[scriptId.DB_ID]){
                    scriptsFieldsNames[scriptId.DB_ID] = [];
                }
                widgetFix.currentRow["SCRIPT_" + scriptId.ID] = scriptId.SCRIPT;
                var scriptField = self.getScriptFromErrorField(scriptId.ID, widgetFix, scriptId.DB_ID);
                if(!scriptsFieldsDBs.includes(scriptId.DB_ID)){
                    scriptsFieldsDBs.push(scriptId.DB_ID);
                    var fieldGroup = self.getFieldGroup(scriptId.DB_ID, scriptId.DB_NAME, widgetFix);
                    fieldGroups.push(fieldGroup);
                }
                widgetFix.fieldGroups = fieldGroups;
                fields.push(scriptField);
                scriptsFieldsNames[scriptId.DB_ID].push(scriptField.name);
                setCodeMirror(false, scriptField.name, widgetFix);
            });
            
            ScreenService.openPopup(widgetFix);

        });
    };

    var setCodeMirror = function(readOnly, campo, widget) {
        var codeMirror = campo ? campo : "CodeMirror";
        campo = campo ? campo : "SCRIPT";
        widget = widget ? widget :getStructureWidget();
        var prepareScriptField = function() {
            var element = $('#'+campo+':visible')[0];
            if(!element) return;
            var editor = CodeMirror.fromTextArea(element, {
                mode: "sql",
                extraKeys: {
                    "Ctrl-Space": "autocomplete",
                    "F11": function (cm) {
                        cm.setOption("fullScreen", !cm.getOption("fullScreen"));
                    },
                    "Esc": function (cm) {
                        if (cm.getOption("fullScreen")) cm.setOption("fullScreen", false);
                    }
                },
                lineNumbers: true,
                theme: "neat",
                placeholder: ScreenService.i18n("Utilize \"/\" como separador de comandos"),
                autoCloseBrackets: true,
                matchBrackets: true,
                readOnly: readOnly
            });
            localStorageEngine.setLocalVar(codeMirror, editor);
            widget.activate();
            clearInterval(interval);
        };
        var interval = setInterval(prepareScriptField, 10);
        var readOnlyLocalVar = readOnly ? 'TRUE' : 'FALSE';
        localStorage.setItem("isReadOnly", readOnlyLocalVar);
    };

    this.structureActionEventOpenUploadWidget = function() {
        var widget = getWidgetScriptUpload();
        ScreenService.openPopup(widget);
        var issueUploadField = getFieldIssuesUplodStructureWidget();
        
            requestDataSourceEngine.getDataSource("/findIssuesRelatedToTask", []).then(function (response) {
                    if(response.dataset && response.dataset.findIssuesRelatedToTask){
                        var issueId = response.dataset.findIssuesRelatedToTask;
                        issueUploadField.value(issueId);
                        Utilities.bugPaginationOnSelectMultiplyOnModeView(issueUploadField).then(function(){
                            widget.redraw();
                        });
                    }
            });
            
            
    };

    this.createFileWebSocket = function (dsDir, structure, dbId) {
        StructureService.downloadByWebSocket(localStorageEngine.getLocalVar('DSEMAIL'), dsDir, structure, dbId).then(function(url) {
            StructureService.removeTempFile(url);
            MessageService.showMessageByCode('SUC_DOWNLOADED', null, { type: 'SUCCESS' });
        }).catch(function (error) {
            if (error.connectionError) {
                MessageService.showCustomDialogByCode('WEB_SOCKET_NOT_FOUND', null, {
                    type: 'CONFIRMATION',
                    actions: [{
                        "label": ScreenService.i18n("Tentar Novamente"),
                        "code": function() {self.createFileWebSocket(dsDir, structure, dbId);}
                    }, {
                        "label": ScreenService.i18n("Download"),
                        "code": function() {StructureService.downloadScript(structure['ID_' + dbId]);}
                    }]
                });
            } else {
                Utilities.showMessageBackEndTrue(error);
            }
        });
    };

    this.validateWebSocketDirectory = function (structure) {
        var repository = localStorage.getItem('repositorio');
        var widget = getStructureWidget();
        return StructureService.validateWebSocketDirectory(repository).then(function(dsDir){
            widget.currentRow.DB_ID.forEach(function(dbId) {
                if(dsDir.DSDIRREPOSITORIO) {
                    self.createFileWebSocket(dsDir.DSDIRREPOSITORIO, structure, dbId);
                } else {
                    StructureService.downloadScript(structure['ID_' + dbId]);
                }
            });
           
        });
    };

    var downloadLog = function(dbId){
        var defer = ZHPromise.defer();
        var widget = getStructureWidget();
        var currentRow = widget.currentRow;
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };
        widget.reload().then(function(){
            StructureService.downloadScriptLog(currentRow.ID, currentRow.DSDESESTRUTURA, widget.currentRow['DSDIRLOG_' + dbId]);
            resolve();
        }).catch(reject);
        return defer.promise;
    };

    this.structureActionEventDownloadLog = function() {
        var widget = getStructureWidget();
        logsDownload = [];
        widget.currentRow.DB_ID.forEach(function(dbId) {
            logsDownload.push(downloadLog.bind(StructureController, dbId));
        });
        Utilities.doSyncMT(logsDownload).then(function(response) {
            widget.reload();
        });
    };

    this.structureActionEventViewLog = function() {
        var widget = getStructureWidget();
        var logWidget = getViewLogWidget();
        logWidget.currentRow = {};
        StructureService.getFullLogs(widget.currentRow).then(function(response){
            if(response){
                widget.currentRow.DB_ID.forEach(function (dbId, index) {
                    if(widget.currentRow['DSDIRLOG_' + dbId]){
                           if(!logWidget.currentRow.LOG){
                                logWidget.currentRow.LOG = response['LOG_' + dbId];
                           }else{
                            logWidget.currentRow.LOG += '\n' + response['LOG_' + dbId];
                           }
                    }
                });

                ScreenService.hideLoader();
                validateActions();
                ScreenService.openPopup(logWidget);
            }else{
                ScreenService.showMessage(ScreenService.i18n("Log inexistente"));
            }
        });

    };

    this.structureViewLogWidgetOnEnter = function() {
        var logWidget = getViewLogWidget();
        var log = logWidget.currentRow.LOG;
        var firstLine = ScreenService.i18n("Design Estrutura:");
        var pos = log.lastIndexOf(firstLine);
        var lastLog = log.substring(pos, log.length);
        logWidget.edit();

        var prepareScriptField = function() {
            var element = $('#LOG:visible')[0];
            if(!element) return;
            var fullHeight = element.scrollHeight;
            element.value = lastLog;
            var cutHeight = element.scrollHeight;
            element.value = log;
            var height = fullHeight - cutHeight;
            element.scrollTo(0, height);
            ScreenService.updateTemplate();
            clearInterval(interval);
        };
        var interval = setInterval(prepareScriptField, 10);
    };

    this.structureActionEventEdit = function() {
        var widget = getStructureWidget();
        widget.edit();
        getRepositoryIdFromEstruturaWidget().reload();
        validateActions();
        if(widget.currentRow.IDRESULTSCRIPT != 'ES' && widget.currentRow.IDRESULTSCRIPT != 'PD'){
            setCodeMirror('nocursor');
        }else{
            setCodeMirror(false);
        }
    };

    var validateActions = function (){
        var widget = getStructureWidget();
        
        var action = getActionParsefromStructureWidget();
        var logAction = getActionBaixarLogfromStructureWidget();
        var scriptAction = getActionBaixarScriptfromStructureWidget();
        var errParse = getActionErrEditStructureWidget();
        var viewLog = getActionViewLogStructureWidget();
        var edit = getActionEditStructureWidget();
        var exec = getActionExecuteStructureWidget();
        var saveDesc = getActionSaveDescStructureWidget();
        var logErrEditAction = getActionErrEditViewLogWIdget();
        var logExecAction = getActionExecViewLogWIdget();
        var inactivateScript = getActionInactivateScript();
        var isActive = widget.currentRow.IDATIVO;
        var remove = getActionRemoveStructureWidget();
        var description = getDescriptionFromStructureWidget();
        var idativo = getIdAtivoFromStructureWidget();

        // variaveis de controle
        var parseError = false;
        var parseSucc = false;
        var execErr = false;
        var execSucc = false;
        var status = widget.currentRow.IDRESULTSCRIPT;
            if(widget.currentRow.DB_ID){
                if(!Array.isArray(widget.currentRow.DB_ID)){
                    widget.currentRow.DB_ID = [widget.currentRow.DB_ID];
                }
                widget.currentRow.DB_ID.forEach(function(dbId) {
                    status = widget.currentRow['IDRESULTSCRIPT_' + dbId] ;
                    
                    if(status == 'PE'){
                        parseError = true;
                    }else if(status == 'PS'){
                        parseSucc = true;
                    }else if(status == 'EE'){
                        execErr = true;
                    } else if(status == 'ES'){
                        execSucc = true;
                    }
                });
            }else{
                status = 'PD';
                parseError = false;
                parseSucc = false;
                execErr = false;
                execSucc = false;
                
            }
            
    
            remove.isVisible = (execErr || execSucc);
            description.readOnly = (status == 'ES');
            idativo.readOnly = (status == 'ES');
            if(isActive == 'N') {
                edit.isVisible = false;
                action.isVisible = false;
                exec.isVisible = false;
                errParse.isVisible = false;
                inactivateScript.isVisible = false;
                
            } else {
                if (logErrEdit) {
                    logErrEditAction.isVisible = true;
                    logExecAction.isVisible = false;
                } else if (logExec) {
                    logExecAction.isVisible = true;
                    logErrEditAction.isVisible = false;
                } else {
                    logErrEditAction.isVisible = false;
                    logExecAction.isVisible = false;
                }
    
                saveDesc.isVisible = (!isFromErr && widget.currentRow.ID);
                action.isVisible = (!widget.currentRow.ID || (parseError && isFromErr));
                logExecAction.isVisible = exec.isVisible = (parseSucc && !parseError || (execErr && isFromErr));
                errParse.isVisible = widget.mode != "edit" && (parseError || execErr);
                scriptAction.isVisible = inactivateScript.isVisible = (execSucc && !execErr);
    
                if (widget.currentRow.ID && widget.mode != "edit") {
                    logAction.isVisible = true;
                    viewLog.isVisible = true;
                } else {
                    logAction.isVisible = false;
                    viewLog.isVisible = false;
                }
            }

        
    };

    var createTemporaryFields = function(name, dbId, widget, fields, fieldGroups){
        
            var fieldGroup = self.getFieldGroup(dbId, name, widget);
            var scriptNumberField = self.getScriptNumberField(dbId, widget);
            var scriptIdField = self.getScriptIdField(dbId, widget);
            var scriptField = self.getScriptField(dbId, widget, true);
            var scriptResultField = self.getScriptResultField(dbId, widget);
            fields.push(scriptIdField);
            fields.push(scriptNumberField);
            fields.push(scriptResultField);
            fields.push(scriptField);
            fieldGroups.push(fieldGroup);
    };

    this.structureWidgetAfterMoveRow = function (){
        var issueField = getFieldIssuesStructureWidget();
        var widget     = getStructureWidget();
        var dbId = widget.currentRow.DB_ID;
        
        removeAllFieldsMtGeneration(widget);
        removeAllFieldsGroupsMtGeneration(widget);
        var fields = widget.fields;
        var fieldGroups = [];

        if(widget.currentRow.ID){
            var name = widget.currentRow.DB_NAME;
            if(widget.currentRow.NRESTRUTREL){
                StructureService.getRelatedScripts(widget.currentRow.NRESTRUTREL).then(function(response){
                    Object.assign(widget.currentRow, response);
                    response.DB_ID.forEach(function (dbId, index) {
                        name = widget.currentRow.DB_NAMES[index];
                        createTemporaryFields(name, dbId, widget, fields, fieldGroups);
                            
                    });
                    fieldGroups.push(self.getAuditFieldGroup());
                    widget.fieldGroups = fieldGroups;
                    widget.fields = fields;
                    validateActions();
                    Utilities.bugPaginationOnSelectMultiplyOnModeView(issueField);
                });
            }else {
                createTemporaryFields(name, dbId, widget, fields, fieldGroups);
                widget.currentRow['NRSCRIPT_' + dbId] = widget.currentRow.DB_ID;
                widget.currentRow['IDRESULTSCRIPT_' + dbId] = widget.currentRow.IDRESULTSCRIPT;
                widget.currentRow['SCRIPT_' + dbId] = widget.currentRow.SCRIPT;
                widget.currentRow['ID_' + dbId] = widget.currentRow.ID;
                widget.currentRow['DSDIRLOG_' + dbId] = widget.currentRow.DSDIRLOG;
                widget.currentRow.DB_ID = [widget.currentRow.DB_ID];
                fieldGroups.push(self.getAuditFieldGroup());
                widget.fieldGroups = fieldGroups;
                widget.fields = fields;
                validateActions();
                Utilities.bugPaginationOnSelectMultiplyOnModeView(issueField);
            }
        }
        
        
    };

    this.repositoryWidgetOnEnter = function() {
        getRepositoryIdFromRepositoryWidget().reload();
    };

    this.structureActionEventDownloadScript = function() {
        var widget     = getStructureWidget();
        var currentRow = widget.currentRow;
        StructureService.downloadScript(currentRow.ID);
    };

    this.structureActionEventDelete = function() {
        var widget     = getStructureWidget();
        MessageService.confirmMessageByCode('ALE_CONFIRM_REMOVE_REGISTER', null,
            {
                type: 'QUESTION',
                success: function() {
                    StructureService.getRelatedScripts(widget.currentRow.ID).then(function(response){
                        Object.assign(widget.currentRow, response);
                        Utilities.genericDeleteEvent(getStructureWidget()).then(function() {
                            MessageService.successNotificationByCode('SUC_DELETED');
                        }).catch(Utilities.showMessageErrorOfBackEnd);
                    });
                   
                },
                fail: function () {}
            });
    };

    this.structureActionEventUpload = function() {
        var widget = getWidgetScriptUpload();
        var logField   = getLogFromStructureWidget();
        var estrutura = getStructureWidget();
        var repository = localStorage.getItem('repositorio');
        if(widget.isValid() && widget.currentRow.DOCUMENT && widget.currentRow.ISSUES_UPLOAD){
            try{
                var script = Utilities.b64Decode(widget.currentRow.DOCUMENT[0].b64File.split(",").pop());
                StructureService.uploadScripts(script, logField, widget.currentRow, repository).then(function(){
                    ScreenService.closePopup(0);
                    MessageService.successNotificationByCode('SUC_UPLOADED');
                    estrutura.reload();
                }).catch(Utilities.showMessageErrorOfBackEnd);
            } catch (e) {
                MessageService.showMessageByCode('ERR_UPLOAD');
            }


        }else {
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
    };
    

    /* --------------------------------------------- CONTAINER FUNCTIONS -------------------------------------------- */
    this.StructureContainerAfterinit = function () {
        var widget = (getStructureWidget());
            localStorageEngine.getLocalVar('loginPromise').then(function () {
                widget.activate();
                WebSocketClient.connectInServer();
            });
        };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('StructureController', StructureController);
});

// FILE: assets/js/Controller/Table.js
function TableController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities, ZhFilterPreferences){

    // ------------------------ GET WIDGETS ------------------------ \\

    var getTableWidget = function () {
        return templateManager.container.getWidget('TABLE');
    };
    var getTableTabWidget = function () {
        return templateManager.container.getWidget('TABLE_TAB');
    };
    var getColumnWidget = function () {
        return templateManager.container.getWidget('COLUMN');
    };
    var getConstraintWidget = function () {
        return templateManager.container.getWidget('CONSTRAINT');
    };
    var getTriggerWidget = function () {
        return templateManager.container.getWidget('TRIGGER');
    };
    var getConstraintColumnsWidget = function () {
        return templateManager.container.getWidget('CONSTRAINT_COLUMNS');
    };

    // ------------------------ Get Fields ------------------------ //

    var getFieldTabelaRelacionadaFromgetConstraintColumnsWidget = function(){
        return getConstraintColumnsWidget().getField('R_TABLE_NAME');
    };

    var getFieldColunaRelacionadaFromgetConstraintColumnsWidget = function(){
        return getConstraintColumnsWidget().getField('R_COLUMN_NAME');
    };

    // ------------------------ END ------------------------ //

    this.tableBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.tableWidgetAfterMoveRow = function(){
        var widget = getTableWidget();
        localStorageEngine.setLocalVar('table_name', widget.currentRow.TABLE_NAME);
        ObjectsService.loadTableObservations(widget.currentRow.TABLE_NAME).then(function(response){

            widget.currentRow.OBSERVATION = response;
        });
    };

    this.tableColumnWidgetAfterMoveRow = function(){
        var widget = getColumnWidget();
        ObjectsService.loadTableColumnObservations(widget.currentRow.COLUMN_NAME, widget.currentRow.TABLE_NAME).then(function(response){
            widget.currentRow.OBSERVATION = response;
        });
    };

    this.conColumnWidgetOnEnter = function(){
        getConstraintColumnsWidget().reload();
    };

    this.tableWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getTableWidget();
            ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };
    this.tableBasicEditAction= function() {
        var widget = getTableWidget();
        localStorageEngine.setLocalVar('tableObservation', widget.currentRow.OBSERVATION);
        widget.edit();

    };

    this.constraintBasicEditAction= function() {
        var widget = getConstraintWidget();
        localStorageEngine.setLocalVar('constraintObservation', widget.currentRow.OBSERVATION);
        widget.edit();

    };

    this.constraintBasicCancelAction= function() {
        getConstraintWidget().currentRow.OBSERVATION =  localStorageEngine.getLocalVar('constraintObservation');
        getConstraintWidget().view();
    };

    this.constraintActionSaveObservation = function (){
        var widget = getConstraintWidget();
        ObjectsService.saveObjectsObservation(widget, 'C', widget.currentRow.CONSTRAINT_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

    this.tableBasicCancelAction= function() {
        getTableWidget().currentRow.OBSERVATION =  localStorageEngine.getLocalVar('tableObservation');
        getTableWidget().view();
    };

    this.columnBasicEditAction= function() {
        var widget = getColumnWidget();
        localStorageEngine.setLocalVar('columnObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.columnBasicCancelAction= function() {
        getColumnWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('columnObservation');
        getColumnWidget().view();
    };


    this.columnWidgetOnEnter = function(){
        getColumnWidget().reload();
    };
    this.triggerWidgetOnEnter = function(){
        getTriggerWidget().reload();
    };

    this.triggerTabWidgetAfterMoveRow = function(){
        var widget = getTriggerWidget();
        var name = widget.currentRow.TRIGGER_NAME;
        var type = 'TRIGGER';
        ObjectsService.getObjectsBody(name, type, 'T').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.constraintWidgetOnEnter = function(){
        getConstraintWidget().reload();
    };

    this.tableActionSaveObservation = function(){
        var widget = getTableWidget();
        ObjectsService.saveTableObservation(widget).then(function(response){
           widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);
    };


    this.tableActionSaveColumnObservation = function(){
        var widget = getColumnWidget();
        ObjectsService.saveTableColumnObservation(widget).then(function(response){
           widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);
    };

    this.constraintTabWidgetAfterMoveRow = function(){
        var widget = getConstraintWidget();
        var tipo =  widget.currentRow.CONSTRAINT_TYPE;
        var type =  "CONSTRAINT";
        var name = widget.currentRow.CONSTRAINT_NAME;
        var constraintWidget = getConstraintColumnsWidget();
        var tabelaR = getFieldTabelaRelacionadaFromgetConstraintColumnsWidget();
        var colunaR = getFieldColunaRelacionadaFromgetConstraintColumnsWidget();
        if(tipo === 'R') {
            constraintWidget.dataSourceFilter[1] =
                {
                    "name": "FK",
                    "operator": "=",
                    "value": "S"
                };
            tabelaR.showOnList = true;
            colunaR.showOnList = true;
        } else {
            constraintWidget.dataSourceFilter[1] = {
                "name": "FK",
                "operator": "=",
                "value": "N"
            };
            tabelaR.showOnList = false;
            colunaR.showOnList = false;
        }
        ObjectsService.getObjectsBody(name, type, 'C').then(function(objectBody){
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.tableWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getTableWidget());
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('TableController', TableController);
});

// FILE: assets/js/Controller/Trigger.js
function TriggerController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getTriggerWidget = function () {
        return templateManager.container.getWidget('TRIGGER');
    };
    // --------------------------- END -------------------------------//

    this.triggerWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getTriggerWidget();
            //ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.triggerBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.triggerBasicEditAction = function(){
        var widget = getTriggerWidget();
        localStorageEngine.setLocalVar('triggerObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.triggerBasicCancelAction = function(){
        getTriggerWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('triggerObservation');
        getTriggerWidget().view();
    };

    this.triggerWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getTriggerWidget());
    };

    this.triggerWidgetAfterMoveRow = function (){
        var widget = getTriggerWidget();
        var name = widget.currentRow.TRIGGER_NAME;
        var type = 'TRIGGER';
        ObjectsService.getObjectsBody(name, type, 'T').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.triggerActionSaveObservation = function (){
        var widget = getTriggerWidget();
        ObjectsService.saveObjectsObservation(widget, 'T', widget.currentRow.TRIGGER_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('TriggerController', TriggerController);
});

// FILE: assets/js/Controller/View.js
function ViewController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getViewWidget = function () {
        return templateManager.container.getWidget('VIEW');
    };

    var getViewColumnsWidget = function () {
        return templateManager.container.getWidget('VIEW_COLUMNS');
    };
    // --------------------------- END -------------------------------//

    this.viewWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getViewWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.viewBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.viewWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getViewWidget());
    };

    this.viewBasicEditAction = function (){
        var widget = getViewWidget();
        localStorageEngine.setLocalVar('viewObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.viewBasicCancelAction = function (){
        getViewWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('viewObservation');
        getViewWidget().view();
    };

    this.viewWidgetAfterMoveRow = function (){
        var widget = getViewWidget();
        var name = widget.currentRow.VIEW_NAME;
        var type = 'VIEW';
        ObjectsService.getObjectsBody(name, type, 'V').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.columnWidgetAfterMoveRow = function (){
        var widget = getViewColumnsWidget();
        ObjectsService.loadTableColumnObservations(widget.currentRow.COLUMN_NAME, widget.currentRow.TABLE_NAME).then(function(response){
            widget.currentRow.OBSERVATION = response;
        });
    };


    this.viewActionSaveObservation = function (){
        var widget = getViewWidget();
        ObjectsService.saveObjectsObservation(widget, 'V', widget.currentRow.VIEW_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

    this.viewColumnWidgetOnEnter = function (){
        getViewColumnsWidget().reload();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewController', ViewController);
});